from DynamicLibraryWithoutArgspec import DynamicLibraryWithoutArgspec


class DynamicLibraryWithKwargsSupportWithoutArgspec(DynamicLibraryWithoutArgspec):

    def run_keyword(self, name, args, kwargs):
        return getattr(self, name)(*args, **kwargs)

    def do_something_with_kwargs(self, a, b=2, c=3, **kwargs):
        print(a, b, c, ' '.join('%s:%s' % (k, v) for k, v in kwargs.items()))



import sys

ROBOT_LISTENER_API_VERSION = 2

def start_test(name, attrs):
    template = attrs['template']
    expected = attrs['doc']
    if template != expected:
        sys.__stderr__.write("Expected template '%s' but got '%s'.\n"
                             % (expected, template))

end_test = start_test



import unittest

from robot.utils import eq, Matcher, MultiMatcher
from robot.utils.asserts import assert_equal, assert_raises


class TestEq(unittest.TestCase):

    def test_eq(self):
        assert eq("foo", "foo")
        assert eq("f OO\t\n", "  foo", caseless=True, spaceless=True)
        assert eq("-a-b-c-", "b", ignore=("-", "a", "c"))
        assert not eq("foo", "bar")
        assert not eq("foo", "FOO", caseless=False)
        assert not eq("foo", "foo ", spaceless=False)


class TestMatcher(unittest.TestCase):

    def test_matcher(self):
        matcher = Matcher('F *', ignore=['-'], caseless=False, spaceless=True)
        assert matcher.pattern == 'F *'
        assert matcher.match('Foo')
        assert matcher.match('--Foo')
        assert not matcher.match('foo')

    def test_regexp_matcher(self):
        matcher = Matcher('F .*', ignore=['-'], caseless=False, spaceless=True,
                          regexp=True)
        assert matcher.pattern == 'F .*'
        assert matcher.match('Foo')
        assert matcher.match('--Foo')
        assert not matcher.match('foo')

    def test_matches_with_string(self):
        for pattern in ['abc', 'ABC', '*', 'a*', '*C', 'a*c', '*a*b*c*', 'AB?',
                        '???', '?b*', '*abc', 'abc*', '*abc*']:
            self._matches('abc', pattern)
        for pattern in ['def', '?abc', '????', '*ed', 'b*']:
            self._matches_not('abc', pattern)

    def test_regexp_matches_with_string(self):
        for pattern in ['abc', 'ABC', '.*', 'a.*', '.*C', 'a.*c', '.*a.*b.*c.*',
                        'AB.',
                        '...', '.b.*', '.*abc', 'abc.*', '.*abc.*']:
            self._matches('abc', pattern, regexp=True)
        for pattern in ['def', '.abc', '....', '.*ed', 'b.*']:
            self._matches_not('abc', pattern, regexp=True)

    def test_matches_with_multiline_string(self):
        for pattern in ['*', 'multi*string', 'multi?line?string', '*\n*']:
            self._matches('multi\nline\nstring', pattern, spaceless=False)

    def test_regexp_matches_with_multiline_string(self):
        for pattern in ['.*', 'multi.*string', 'multi.line.string', '.*\n.*']:
            self._matches('multi\nline\nstring', pattern, spaceless=False,
                          regexp=True)

    def test_matches_with_slashes(self):
        for pattern in ['a*','aa?b*','*c','?a?b?c']:
            self._matches('aa/b\\c', pattern)

    def test_regexp_matches_with_slashes(self):
        for pattern in ['a.*', 'aa.b.*', '.*c', '.a.b.c']:
            self._matches('aa/b\\c', pattern, regexp=True)

    def test_matches_no_pattern(self):
        for string in ['foo', '', ' ', '      ', 'what ever',
                       'multi\nline\nstring here', '=\\.)(/23.',
                       'forw/slash/and\\back\\slash']:
            self._matches(string, string), string

    def test_regexp_matches_no_pattern(self):
        for string in ['foo', '', ' ', '      ', 'what ever']:
            self._matches(string, string, regexp=True), string

    def test_match_any(self):
        matcher = Matcher('H?llo')
        assert matcher.match_any(('Hello', 'world'))
        assert matcher.match_any(['jam', 'is', 'hillo'])
        assert not matcher.match_any(('no', 'match', 'here'))
        assert not matcher.match_any(())

    def test_regexp_match_any(self):
        matcher = Matcher('H.llo', regexp=True)
        assert matcher.match_any(('Hello', 'world'))
        assert matcher.match_any(['jam', 'is', 'hillo'])
        assert not matcher.match_any(('no', 'match', 'here'))
        assert not matcher.match_any(())

    def test_bytes(self):
        assert_raises(TypeError, Matcher, b'foo')
        assert_raises(TypeError, Matcher('foo').match, b'foo')

    def test_glob_sequence(self):
        pattern = '[Tre]est [CR]at'
        self._matches('Test Cat', pattern)
        self._matches('Rest Rat', pattern)
        self._matches('rest Rat', pattern, caseless=False)
        self._matches_not('rest rat', pattern, caseless=False)
        self._matches_not('Test Bat', pattern)
        self._matches_not('Best Bat', pattern)

    def test_glob_sequence_negative(self):
        pattern = '[!Tre]est [!CR]at'
        self._matches_not('Test Bat', pattern)
        self._matches_not('Best Rat', pattern)
        self._matches('Best Bat', pattern)

    def test_glob_range(self):
        pattern = 'GlobTest[1-2]'
        self._matches('GlobTest1', pattern)
        self._matches('GlobTest2', pattern)
        self._matches_not('GlobTest3', pattern)

    def test_glob_range_negative(self):
        pattern = 'GlobTest[!1-2]'
        self._matches_not('GlobTest1', pattern)
        self._matches_not('GlobTest2', pattern)
        self._matches('GlobTest3', pattern)

    def test_escape_wildcards(self):
        # No escaping needed
        self._matches('[', '[')
        self._matches('[]', '[]')
        # Escaping needed
        self._matches_not('[x]', '[x]')
        self._matches('[x]', '[[]x]')
        for wild in '*?[]':
            self._matches(wild, '[%s]' % wild)
            self._matches('foo%sbar' % wild, 'foo[%s]bar' % wild)
            self._matches('foo%sbar' % wild, '*[%s]???' % wild)

    def test_spaceless(self):
        for text in ['fbar', 'foobar']:
            assert Matcher('f*bar').match(text)
            assert Matcher('f * b a r').match(text)
            assert Matcher('f*bar', spaceless=False).match(text)
        for text in ['f b a r', 'f o o b a r', '   foo bar   ', 'fbar\n']:
            assert Matcher('f*bar').match(text)
            assert not Matcher('f*bar', spaceless=False).match(text)

    def _matches(self, string, pattern, **config):
        assert Matcher(pattern, **config).match(string), pattern

    def _matches_not(self, string, pattern, **config):
        assert not Matcher(pattern, **config).match(string), pattern


class TestMultiMatcher(unittest.TestCase):

    def test_match_pattern(self):
        matcher = MultiMatcher(['xxx', 'f*'], ignore='.:')
        assert matcher.match('xxx')
        assert matcher.match('foo')
        assert matcher.match('..::FOO::..')
        assert not matcher.match('bar')

    def test_match_regexp_pattern(self):
        matcher = MultiMatcher(['xxx', 'f.*'], ignore='_:', regexp=True)
        assert matcher.match('xxx')
        assert matcher.match('foo')
        assert matcher.match('__::FOO::__')
        assert not matcher.match('bar')

    def test_do_not_match_when_no_patterns_by_default(self):
        assert not MultiMatcher().match('xxx')

    def test_configure_to_match_when_no_patterns(self):
        assert MultiMatcher(match_if_no_patterns=True).match('xxx')
        assert MultiMatcher(match_if_no_patterns=True, regexp=True).match('xxx')

    def test_len(self):
        assert_equal(len(MultiMatcher()), 0)
        assert_equal(len(MultiMatcher([])), 0)
        assert_equal(len(MultiMatcher(['one', 'two'])), 2)
        assert_equal(len(MultiMatcher(regexp=True)), 0)
        assert_equal(len(MultiMatcher([], regexp=True)), 0)
        assert_equal(len(MultiMatcher(['one', 'two'], regexp=True)), 2)

    def test_iter(self):
        assert_equal(tuple(MultiMatcher()), ())
        assert_equal([m.pattern for m in MultiMatcher(['1', 'xxx', '3'])],
                     ['1', 'xxx', '3'])
        assert_equal(tuple(MultiMatcher(regexp=True)), ())
        assert_equal([m.pattern for m in MultiMatcher(['1', 'xxx', '3'], regexp=True)],
                     ['1', 'xxx', '3'])

    def test_single_string_is_converted_to_list(self):
        matcher = MultiMatcher('one string')
        assert matcher.match('one string')
        assert not matcher.match('o')
        assert_equal(len(matcher), 1)

    def test_regexp_single_string_is_converted_to_list(self):
        matcher = MultiMatcher('one string', regexp=True)
        assert matcher.match('one string')
        assert not matcher.match('o')
        assert_equal(len(matcher), 1)

    def test_match_any(self):
        matcher = MultiMatcher(['H?llo', 'w*'])
        assert matcher.match_any(('Hi', 'world'))
        assert matcher.match_any(['jam', 'is', 'hillo'])
        assert not matcher.match_any(('no', 'match', 'here'))
        assert not matcher.match_any(())

    def test_regexp_match_any(self):
        matcher = MultiMatcher(['H.llo', 'w.*'], regexp=True)
        assert matcher.match_any(('Hi', 'world'))
        assert matcher.match_any(['jam', 'is', 'hillo'])
        assert not matcher.match_any(('no', 'match', 'here'))
        assert not matcher.match_any(())


if __name__ == '__main__':
    unittest.main()



import sys

from xmlrpc.client import Binary

from remoteserver import RemoteServer, keyword


class TypedRemoteServer(RemoteServer):

    def _register_functions(self):
        RemoteServer._register_functions(self)
        self.register_function(self.get_keyword_types)

    def get_keyword_types(self, name):
        kw = getattr(self.library, name)
        return getattr(kw, 'robot_types', None)

    def get_keyword_arguments(self, name):
        if name == 'defaults_as_tuples':
            return [('first', 'eka'), ('second', 2)]
        return RemoteServer.get_keyword_arguments(self, name)


class Arguments:

    def argument_should_be(self, argument, expected, binary=False):
        if binary:
            argument = self._handle_binary(argument)
        expected = eval(expected)
        self._assert_equal(argument, expected)

    def _assert_equal(self, argument, expected, msg=None):
        assert argument == expected, msg or '%r != %r' % (argument, expected)

    def _handle_binary(self, arg, required=True):
        if isinstance(arg, list):
            return self._handle_binary_in_list(arg)
        if isinstance(arg, dict):
            return self._handle_binary_in_dict(arg)
        assert isinstance(arg, Binary) or not required, 'Non-binary argument'
        return arg.data if isinstance(arg, Binary) else arg

    def _handle_binary_in_list(self, arg):
        assert any(isinstance(a, Binary) for a in arg), 'No binary in list'
        return [self._handle_binary(a, required=False) for a in arg]

    def _handle_binary_in_dict(self, arg):
        assert any(isinstance(key, Binary) or isinstance(value, Binary)
                   for key, value in arg.items()), 'No binary in dict'
        return dict((self._handle_binary(key, required=False),
                     self._handle_binary(value, required=False))
                    for key, value in arg.items())

    def kwarg_should_be(self, **kwargs):
        self.argument_should_be(**kwargs)

    def no_arguments(self):
        return self._format_args()

    def one_argument(self, arg):
        return self._format_args(arg)

    def two_arguments(self, arg1, arg2):
        return self._format_args(arg1, arg2)

    def five_arguments(self, arg1, arg2, arg3, arg4, arg5):
        return self._format_args(arg1, arg2, arg3, arg4, arg5)

    def arguments_with_default_values(self, arg1, arg2=2, arg3='3'):
        return self._format_args(arg1, arg2, arg3)

    def varargs(self, *args):
        return self._format_args(*args)

    def required_defaults_and_varargs(self, req, default='world', *varargs):
        return self._format_args(req, default, *varargs)

    # Handled separately by get_keyword_arguments above.
    def defaults_as_tuples(self, first='eka', second=2):
        return self._format_args(first, second)

    def kwargs(self, **kwargs):
        return self._format_args(**kwargs)

    def kw_only_arg(self, *, kwo):
        return self._format_args(kwo=kwo)

    def kw_only_arg_with_default(self, *, k1='default', k2):
        return self._format_args(k1=k1, k2=k2)

    def args_and_kwargs(self, arg1='default1', arg2='default2', **kwargs):
        return self._format_args(arg1, arg2, **kwargs)

    def varargs_and_kwargs(self, *varargs, **kwargs):
        return self._format_args(*varargs, **kwargs)

    def all_arg_types(self, arg1, arg2='default', *varargs,
                      kwo1='default', kwo2, **kwargs):
        return self._format_args(arg1, arg2, *varargs,
                                 kwo1=kwo1, kwo2=kwo2, **kwargs)

    @keyword(types=['int', '', 'dict'])
    def argument_types_as_list(self, integer, no_type_1, dictionary, no_type_2):
        self._assert_equal(integer, 42)
        self._assert_equal(no_type_1, '42')
        self._assert_equal(dictionary, {'a': 1, 'b': 'ä'})
        self._assert_equal(no_type_2, '{}')

    @keyword(types={'integer': 'Integer', 'dictionary': 'Dictionary'})
    def argument_types_as_dict(self, integer, no_type_1, dictionary, no_type_2):
        self.argument_types_as_list(integer, no_type_1, dictionary, no_type_2)

    def _format_args(self, *args, **kwargs):
        args = [self._format(a) for a in args]
        kwargs = [f'{k}:{self._format(kwargs[k])}' for k in sorted(kwargs)]
        return ', '.join(args + kwargs)

    def _format(self, arg):
        type_name = type(arg).__name__
        return arg if isinstance(arg, str) else f'{arg} ({type_name})'


if __name__ == '__main__':
    TypedRemoteServer(Arguments(), *sys.argv[1:])



import sys
import time
from remoteserver import RemoteServer


class Timeouts:

    def sleep(self, secs):
        time.sleep(int(secs))


if __name__ == '__main__':
    RemoteServer(Timeouts(), *sys.argv[1:])



from collections.abc import Mapping


class MyObject:

    def __init__(self, name='<MyObject>'):
        self.name = name

    def __str__(self):
        return self.name


class MyMapping(Mapping):

    def __init__(self, data=None, **extra):
        self.data = data or {}
        self.data.update(extra)

    def __getitem__(self, item):
        return self.data[item]

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        return iter(self.data)



from io import StringIO
import unittest

from robot.utils import HtmlWriter
from robot.utils.asserts import assert_equal


class TestHtmlWriter(unittest.TestCase):

    def setUp(self):
        self.output = StringIO()
        self.writer = HtmlWriter(self.output)

    def test_start(self):
        self.writer.start('r')
        self._verify('<r>\n')

    def test_start_without_newline(self):
        self.writer.start('robot', newline=False)
        self._verify('<robot>')

    def test_start_with_attribute(self):
        self.writer.start('robot', {'name': 'Suite1'}, False)
        self._verify('<robot name="Suite1">')

    def test_start_with_attributes(self):
        self.writer.start('test', {'class': '123', 'x': 'y', 'a': 'z'})
        self._verify('<test a="z" class="123" x="y">\n')

    def test_start_with_non_ascii_attributes(self):
        self.writer.start('test', {'name': u'\xA7', u'\xE4': u'\xA7'})
        self._verify(u'<test name="\xA7" \xE4="\xA7">\n')

    def test_start_with_quotes_in_attribute_value(self):
        self.writer.start('x', {'q':'"', 'qs': '""""', 'a': "'"}, False)
        self._verify('<x a="\'" q="&quot;" qs="&quot;&quot;&quot;&quot;">')

    def test_start_with_html_in_attribute_values(self):
        self.writer.start('x', {'1':'<', '2': '&', '3': '</html>'}, False)
        self._verify('<x 1="&lt;" 2="&amp;" 3="&lt;/html&gt;">')

    def test_start_with_newlines_and_tabs_in_attribute_values(self):
        self.writer.start('x', {'1':'\n', '3': 'A\nB\tC', '2': '\t', '4': '\r\n'}, False)
        self._verify('<x 1="&#10;" 2="&#09;" 3="A&#10;B&#09;C" 4="&#13;&#10;">')

    def test_end(self):
        self.writer.start('robot', newline=False)
        self.writer.end('robot')
        self._verify('<robot></robot>\n')

    def test_end_without_newline(self):
        self.writer.start('robot', newline=False)
        self.writer.end('robot', newline=False)
        self._verify('<robot></robot>')

    def test_end_alone(self):
        self.writer.end('suite', newline=False)
        self._verify('</suite>')

    def test_content(self):
        self.writer.start('robot')
        self.writer.content('Hello world!')
        self._verify('<robot>\nHello world!')

    def test_content_with_non_ascii_data(self):
        self.writer.start('robot', newline=False)
        self.writer.content(u'Circle is 360\xB0. ')
        self.writer.content(u'Hyv\xE4\xE4 \xFC\xF6t\xE4!')
        self.writer.end('robot', newline=False)
        expected = u'Circle is 360\xB0. Hyv\xE4\xE4 \xFC\xF6t\xE4!'
        self._verify('<robot>%s</robot>' % expected)

    def test_multiple_content(self):
        self.writer.start('robot')
        self.writer.content('Hello world!')
        self.writer.content('Hi again!')
        self._verify('<robot>\nHello world!Hi again!')

    def test_content_with_chars_needing_escaping(self):
        self.writer.content('Me, "Myself" & I > U')
        self._verify('Me, "Myself" &amp; I &gt; U')

    def test_content_alone(self):
        self.writer.content('hello')
        self._verify('hello')

    def test_none_content(self):
        self.writer.start('robot')
        self.writer.content(None)
        self.writer.content('')
        self._verify('<robot>\n')

    def test_element(self):
        self.writer.element('div', 'content', {'id': '1'})
        self.writer.element('i', newline=False)
        self._verify('<div id="1">content</div>\n<i></i>')

    def test_line_separator(self):
        output = StringIO()
        writer = HtmlWriter(output)
        writer.start('b')
        writer.end('b')
        writer.element('i')
        assert_equal(output.getvalue(), '<b>\n</b>\n<i></i>\n')

    def test_non_ascii(self):
        self.output = StringIO()
        writer = HtmlWriter(self.output)
        writer.start(u'p', attrs={'name': u'hyv\xe4\xe4'}, newline=False)
        writer.content(u'y\xf6')
        writer.element('i', u't\xe4', newline=False)
        writer.end('p', newline=False)
        self._verify(u'<p name="hyv\xe4\xe4">y\xf6<i>t\xe4</i></p>')

    def _verify(self, expected):
        assert_equal(self.output.getvalue(), expected)


if __name__ == '__main__':
    unittest.main()



class Listener:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self):
        self.start_kw_count = 0
        self.end_kw_count = 0

    def start_keyword(self, kw, attrs):
        self.start_kw_count += 1

    def end_keyword(self, kw, attrs):
        self.end_kw_count += 1

    def end_suite(self, *args):
        if not self.start_kw_count:
            raise AssertionError("No keywords started")
        if not self.end_kw_count:
            raise AssertionError("No keywords ended")



import unittest

from robot.utils.frange import frange, _digits
from robot.utils.asserts import assert_equal, assert_true, assert_raises


class TestFrange(unittest.TestCase):

    def test_basics(self):
        for input, expected in [([6.0], [0.0, 1.0, 2.0, 3.0, 4.0, 5.0]),
                                ([6.01], [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0]),
                                ([-2.4, 2.1], [-2.4, -1.4, -0.4, 0.6, 1.6]),
                                ([0, 0.5, 0.1], [0, 0.1, 0.2, 0.3, 0.4])]:
            assert_equal(frange(*input), expected)

    def test_numbers_with_e(self):
        for input, expected in [([1e20, 1e21, 2e20], [1e20, 3e20, 5e20, 7e20, 9e20]),
                                ([1e-21, 1.1e-20, 3e-21], [1e-21, 4e-21, 7e-21, 1e-20]),
                                ([1.1e-20, 1.1e-21, -5e-21], [1.1e-20, 6e-21])]:
            result = frange(*input)
            assert_equal(len(result), len(expected))
            # Floats are not accurate and values depend on Python versions
            diffs = [round(r-e, 30) for r, e in zip(result, expected)]
            assert_equal(sum(diffs), 0)

    def test_compatibility_with_range(self):
        for input in [(10,), (-10,), (1, 10), (1, 10, 2), (10, -5, -2)]:
            assert_equal(frange(*input), list(range(*input)))
            assert_equal(frange(*(float(i) for i in input)), list(range(*input)))

    def test_preserve_type(self):
        for input in [(2,), (0, 2), (0, 2, 1)]:
            assert_true(all(isinstance(item, int) for item in frange(*input)))
        for input in [(2.0,), (0, 2.0), (0.0, 2), (0, 2, 1.0), (0, 2.0, 1)]:
            assert_true(all(isinstance(item, float) for item in frange(*input)))

    def test_invalid_args(self):
        assert_raises(TypeError, frange, ())
        assert_raises(TypeError, frange, (1, 2, 3, 4))

    def test_digits(self):
        # Using strings with some values to avoid problems representing floats:
        # - With older Python versions e.g. repr(3.1) == '3.1000000000000001'.
        # - With any version e.g. repr(1.23e3) == '1230.0'
        for input, expected in [(3, 0),
                                (3.0, 0),
                                ('3.1', 1),
                                ('3.14', 2),
                                ('3.141592653589793', len('141592653589793')),
                                (1000.1000, 1),
                                ('-2.458', 3),
                                (1e50, 0),
                                (1.23e50, 0),
                                (1e-50, 50),
                                ('1.23e-50', 52),
                                ('1.23e3', 0),
                                ('1.23e2', 0),
                                ('1.23e1', 1),
                                ('1.23e0', 2),
                                ('1.23e-1', 3),
                                ('1.23e-2', 4)]:
            assert_equal(_digits(input), expected, input)


if __name__ == "__main__":
    unittest.main()



class DocFormat:
    """Library to test documentation formatting.

    *bold* or <b>bold</b> http://example.com
    """

    def keyword(self):
        """*bold* or <b>bold</b> http://example.com"""

    def link(self):
        """Link to `Keyword`."""

    def rest(self):
        """Let's see *how well* reST__ works.

        This documentation is mainly used for manually verifying reST output.
        This link to \\`Keyword\\` is also automatically tested.

        ====  =====
        My    table
        two   rows
        ====  =====

        - list
        - here

        Preformatted::

            def example():
                pass

        __ http://docutils.sourceforge.net

        .. code:: robotframework

            *** Test Cases ***
            Example
                Log    How cool is this!?!?!1!
        """



from robot.libraries.BuiltIn import BuiltIn
from robot.utils import is_string


def get_result_or_error(*args):
    try:
        return BuiltIn().run_keyword(*args)
    except Exception as err:
        return err.message


def pretty(*args, **kwargs):
    args = [to_str(a) for a in args]
    kwargs = ['%s:%s' % (k, to_str(v)) for k, v in sorted(kwargs.items())]
    return ', '.join(args + kwargs)


def to_str(arg):
    if is_string(arg):
        return arg
    return '%s (%s)' % (arg, type(arg).__name__)



class DynamicLibraryWithKeywordTags:

    def get_keyword_names(self):
        return ['dynamic_library_keyword_with_tags']

    def run_keyword(self, name, *args):
        return None

    def get_keyword_documentation(self, name):
        return 'Summary line\nTags: foo, bar'



import os
from threading import Timer


class wait_until_library:

    def __init__(self):
        self._timers = []

    def remove_after_sleeping(self, *paths):
        for p in paths:
            remover = os.rmdir if os.path.isdir(p) else os.remove
            self._run_after_sleeping(remover, p)

    def create_file_after_sleeping(self, path):
        self._run_after_sleeping(lambda: open(path, 'w').close())

    def create_dir_after_sleeping(self, path):
        self._run_after_sleeping(os.mkdir, path)

    def _run_after_sleeping(self, method, *args):
        self._timers.append(Timer(0.2, method, args))
        self._timers[-1].start()



class _Object:
    def __init__(self, name):
        self.name = name
    def __str__(self):
        return self.name
    def __repr__(self):
        return "'%s'" % self.name

STRING = 'Hello world!'
INTEGER = 42
FLOAT = -1.2
BOOLEAN = True
NONE_VALUE = None
ESCAPES = 'one \\ two \\\\ ${non_existing}'
NO_VALUE = ''
LIST = ['Hello', 'world', '!']
LIST_WITH_NON_STRINGS = [42, -1.2, True, None]
LIST_WITH_ESCAPES = ['one \\', 'two \\\\', 'three \\\\\\', '${non_existing}']
OBJECT = _Object('dude')

LIST__ONE_ITEM = ['Hello again?']
LIST__LIST_2 = ['Hello', 'again', '?']
LIST__LIST_WITH_ESCAPES_2 = LIST_WITH_ESCAPES[:]
LIST__EMPTY_LIST = []
LIST__OBJECTS = [STRING, INTEGER, LIST, OBJECT]

lowercase = 'Variable name in lower case'
LIST__lowercase_list = [lowercase]
Und_er__scores_____ = 'Variable name with under scores'
LIST________UN__der__SCO__r_e_s__liST__ = [Und_er__scores_____]


PRIORITIES_1 = PRIORITIES_2 = PRIORITIES_3 = PRIORITIES_4 = PRIORITIES_4B \
               = 'Variable File'



import unittest
import tempfile
import inspect
import shutil
import sys
import os
import re
from os.path import basename, dirname, exists, join, normpath

from robot.errors import DataError
from robot.utils import abspath, WINDOWS
from robot.utils.importer import Importer, ByPathImporter
from robot.utils.asserts import (assert_equal, assert_true, assert_raises,
                                 assert_raises_with_msg)


CURDIR = dirname(abspath(__file__))
LIBDIR = normpath(join(CURDIR, '..', '..', 'atest', 'testresources', 'testlibs'))
TEMPDIR = tempfile.gettempdir()
TESTDIR = join(TEMPDIR, 'robot-importer-testing')
WINDOWS_PATH_IN_ERROR = re.compile(r"'\w:\\")


def assert_prefix(error, expected):
    message = str(error)
    count = 3 if WINDOWS_PATH_IN_ERROR.search(message) else 2
    prefix = ':'.join(message.split(':')[:count]) + ':'
    assert_equal(prefix, expected)


def create_temp_file(name, attr=42, extra_content=''):
    if not exists(TESTDIR):
        os.mkdir(TESTDIR)
    path = join(TESTDIR, name)
    with open(path, 'w') as file:
        file.write('''
attr = %r
def func():
    return attr
''' % attr)
        file.write(extra_content)
    return path


class LoggerStub:

    def __init__(self, remove_extension=False):
        self.messages = []
        self.remove_extension = remove_extension

    def info(self, msg):
        if self.remove_extension:
            for ext in '$py.class', '.pyc', '.py':
                msg = msg.replace(ext, '')
        self.messages.append(self._normalize_drive_letter(msg))

    def assert_message(self, msg, index=0):
        assert_equal(self.messages[index], self._normalize_drive_letter(msg))

    def _normalize_drive_letter(self, msg):
        if not WINDOWS:
            return msg
        return re.sub("'\\w:", lambda match: match.group().upper(), msg)


class TestImportByPath(unittest.TestCase):

    def setUp(self):
        self.tearDown()

    def tearDown(self):
        if exists(TESTDIR):
            shutil.rmtree(TESTDIR)

    def test_python_file(self):
        path = create_temp_file('test.py')
        self._import_and_verify(path, remove='test')
        self._assert_imported_message('test', path)

    def test_python_directory(self):
        create_temp_file('__init__.py')
        module_name = basename(TESTDIR)
        self._import_and_verify(TESTDIR, remove=module_name)
        self._assert_imported_message(module_name, TESTDIR)

    def test_import_same_file_multiple_times(self):
        path = create_temp_file('test.py')
        self._import_and_verify(path, remove='test')
        self._assert_imported_message('test', path)
        self._import_and_verify(path)
        self._assert_imported_message('test', path)
        self._import_and_verify(path, name='library')
        self._assert_imported_message('test', path, type='library module')

    def test_import_different_file_and_directory_with_same_name(self):
        path1 = create_temp_file('test.py', attr=1)
        self._import_and_verify(path1, attr=1, remove='test')
        self._assert_imported_message('test', path1)
        path2 = join(TESTDIR, 'test')
        os.mkdir(path2)
        create_temp_file(join(path2, '__init__.py'), attr=2)
        self._import_and_verify(path2, attr=2, directory=path2)
        self._assert_removed_message('test')
        self._assert_imported_message('test', path2, index=1)
        path3 = create_temp_file(join(path2, 'test.py'), attr=3)
        self._import_and_verify(path3, attr=3, directory=path2)
        self._assert_removed_message('test')
        self._assert_imported_message('test', path3, index=1)

    def test_import_class_from_file(self):
        path = create_temp_file('test.py', extra_content='''
class test:
    def method(self):
        return 42
''')
        klass = self._import(path, remove='test')
        self._assert_imported_message('test', path, type='class')
        assert_true(inspect.isclass(klass))
        assert_equal(klass.__name__, 'test')
        assert_equal(klass().method(), 42)

    def test_invalid_python_file(self):
        path = create_temp_file('test.py', extra_content='invalid content')
        error = assert_raises(DataError, self._import_and_verify, path, remove='test')
        assert_prefix(error, "Importing '%s' failed: SyntaxError:" % path)

    def _import_and_verify(self, path, attr=42, directory=TESTDIR,
                           name=None, remove=None):
        module = self._import(path, name, remove)
        assert_equal(module.attr, attr)
        assert_equal(module.func(), attr)
        if hasattr(module, '__file__'):
            assert_equal(dirname(abspath(module.__file__)), directory)

    def _import(self, path, name=None, remove=None):
        if remove and remove in sys.modules:
            sys.modules.pop(remove)
        self.logger = LoggerStub()
        importer = Importer(name, self.logger)
        sys_path_before = sys.path[:]
        try:
            return importer.import_class_or_module_by_path(path)
        finally:
            assert_equal(sys.path, sys_path_before)

    def _assert_imported_message(self, name, source, type='module', index=0):
        msg = "Imported %s '%s' from '%s'." % (type, name, source)
        self.logger.assert_message(msg, index=index)

    def _assert_removed_message(self, name, index=0):
        msg = "Removed module '%s' from sys.modules to import fresh module." % name
        self.logger.assert_message(msg, index=index)


class TestInvalidImportPath(unittest.TestCase):

    def test_non_existing(self):
        path = 'non-existing.py'
        assert_raises_with_msg(
            DataError,
            "Importing '%s' failed: File or directory does not exist." % path,
            Importer().import_class_or_module_by_path, path
        )
        path = abspath(path)
        assert_raises_with_msg(
            DataError,
            "Importing test file '%s' failed: File or directory does not exist." % path,
            Importer('test file').import_class_or_module_by_path, path
        )

    def test_non_absolute(self):
        path = os.listdir('.')[0]
        assert_raises_with_msg(
            DataError,
            "Importing '%s' failed: Import path must be absolute." % path,
            Importer().import_class_or_module_by_path, path
        )
        assert_raises_with_msg(
            DataError,
            "Importing file '%s' failed: Import path must be absolute." % path,
            Importer('file').import_class_or_module_by_path, path
        )

    def test_invalid_format(self):
        path = join(CURDIR, '..', '..', 'README.rst')
        assert_raises_with_msg(
            DataError,
            "Importing '%s' failed: Not a valid file or directory to import." % path,
            Importer().import_class_or_module_by_path, path
        )
        assert_raises_with_msg(
            DataError,
            "Importing xxx '%s' failed: Not a valid file or directory to import." % path,
            Importer('xxx').import_class_or_module_by_path, path
        )


class TestImportClassOrModule(unittest.TestCase):

    def test_import_module_file(self):
        module = self._import_module('classes')
        assert_equal(module.__version__, 'N/A')

    def test_import_module_directory(self):
        module = self._import_module('pythonmodule')
        assert_equal(module.some_string, 'Hello, World!')

    def test_import_non_existing(self):
        error = assert_raises(DataError, self._import, 'NonExisting')
        assert_prefix(error, "Importing 'NonExisting' failed: ModuleNotFoundError:")

    def test_import_sub_module(self):
        module = self._import_module('pythonmodule.library')
        assert_equal(module.keyword_from_submodule('Kitty'), 'Hello, Kitty!')
        module = self._import_module('pythonmodule.submodule')
        assert_equal(module.attribute, 42)
        module = self._import_module('pythonmodule.submodule.sublib')
        assert_equal(module.keyword_from_deeper_submodule(), 'hi again')

    def test_import_class_with_same_name_as_module(self):
        klass = self._import_class('ExampleLibrary')
        assert_equal(klass().return_string_from_library('xxx'), 'xxx')

    def test_import_class_from_module(self):
        klass = self._import_class('ExampleLibrary.ExampleLibrary')
        assert_equal(klass().return_string_from_library('yyy'), 'yyy')

    def test_import_class_from_sub_module(self):
        klass = self._import_class('pythonmodule.submodule.sublib.Sub')
        assert_equal(klass().keyword_from_class_in_deeper_submodule(), 'bye')

    def test_import_non_existing_item_from_existing_module(self):
        assert_raises_with_msg(DataError,
                               "Importing 'pythonmodule.NonExisting' failed: "
                               "Module 'pythonmodule' does not contain 'NonExisting'.",
                               self._import, 'pythonmodule.NonExisting')
        assert_raises_with_msg(DataError,
                               "Importing test library 'pythonmodule.none' failed: "
                               "Module 'pythonmodule' does not contain 'none'.",
                               self._import, 'pythonmodule.none', 'test library')

    def test_invalid_item_from_existing_module(self):
        assert_raises_with_msg(DataError,
                               "Importing 'pythonmodule.some_string' failed: "
                               "Expected class or module, got string.",
                               self._import, 'pythonmodule.some_string')
        assert_raises_with_msg(DataError,
                               "Importing xxx 'pythonmodule.submodule.attribute' failed: "
                               "Expected class or module, got integer.",
                               self._import, 'pythonmodule.submodule.attribute', 'xxx')

    def test_item_from_non_existing_module(self):
        error = assert_raises(DataError, self._import, 'nonex.item')
        assert_prefix(error, "Importing 'nonex.item' failed: ModuleNotFoundError:")

    def test_import_file_by_path(self):
        import module_library as expected
        module = self._import_module(join(LIBDIR, 'module_library.py'))
        assert_equal(module.__name__, expected.__name__)
        assert_equal(dirname(normpath(module.__file__)),
                     dirname(normpath(expected.__file__)))
        assert_equal(dir(module), dir(expected))

    def test_import_class_from_file_by_path(self):
        klass = self._import_class(join(LIBDIR, 'ExampleLibrary.py'))
        assert_equal(klass().return_string_from_library('test'), 'test')

    def test_invalid_file_by_path(self):
        path = join(TEMPDIR, 'robot_import_invalid_test_file.py')
        try:
            with open(path, 'w') as file:
                file.write('invalid content')
            error = assert_raises(DataError, self._import, path)
            assert_prefix(error, "Importing '%s' failed: SyntaxError:" % path)
        finally:
            os.remove(path)

    def test_logging_when_importing_module(self):
        logger = LoggerStub(remove_extension=True)
        self._import_module('classes', 'test library', logger)
        logger.assert_message("Imported test library module 'classes' from '%s'."
                              % join(LIBDIR, 'classes'))

    def test_logging_when_importing_python_class(self):
        logger = LoggerStub(remove_extension=True)
        self._import_class('ExampleLibrary', logger=logger)
        logger.assert_message("Imported class 'ExampleLibrary' from '%s'."
                              % join(LIBDIR, 'ExampleLibrary'))

    def _import_module(self, name, type=None, logger=None):
        module = self._import(name, type, logger)
        assert_true(inspect.ismodule(module))
        return module

    def _import_class(self, name, type=None, logger=None):
        klass = self._import(name, type, logger)
        assert_true(inspect.isclass(klass))
        return klass

    def _import(self, name, type=None, logger=None):
        return Importer(type, logger or LoggerStub()).import_class_or_module(name)


class TestImportModule(unittest.TestCase):

    def test_import_module(self):
        module = Importer().import_module('ExampleLibrary')
        assert_equal(module.ExampleLibrary().return_string_from_library('xxx'), 'xxx')

    def test_logging(self):
        logger = LoggerStub(remove_extension=True)
        Importer(logger=logger).import_module('ExampleLibrary')
        logger.assert_message("Imported module 'ExampleLibrary' from '%s'."
                              % join(LIBDIR, 'ExampleLibrary'))


class TestErrorDetails(unittest.TestCase):

    def test_no_traceback(self):
        error = self._failing_import('NoneExisting')
        assert_equal(self._get_traceback(error),
                     'Traceback (most recent call last):\n  None')

    def test_traceback(self):
        path = create_temp_file('tb.py', extra_content='import nonex')
        try:
            error = self._failing_import(path)
        finally:
            shutil.rmtree(TESTDIR)
        assert_equal(self._get_traceback(error), '''\
Traceback (most recent call last):
  File "%s", line 5, in <module>
    import nonex''' % path)

    def test_pythonpath(self):
        error = self._failing_import('NoneExisting')
        lines = self._get_pythonpath(error).splitlines()
        assert_equal(lines[0], 'PYTHONPATH:')
        for line in lines[1:]:
            assert_true(line.startswith('  '))

    def test_non_ascii_entry_in_pythonpath(self):
        sys.path.append('hyv\xe4')
        try:
            error = self._failing_import('NoneExisting')
        finally:
            sys.path.pop()
        last_line = self._get_pythonpath(error).splitlines()[-1].strip()
        assert_true(last_line.startswith('hyv'))

    def test_structure(self):
        error = self._failing_import('NoneExisting')
        message = ("Importing 'NoneExisting' failed: ModuleNotFoundError: "
                   "No module named 'NoneExisting'")
        expected = (message, self._get_traceback(error), self._get_pythonpath(error))
        assert_equal(str(error), '\n'.join(expected))

    def _failing_import(self, name):
        importer = Importer().import_class_or_module
        return assert_raises(DataError, importer, name)

    def _get_traceback(self, error):
        return '\n'.join(self._block(error, 'Traceback (most recent call last):',
                                     'PYTHONPATH:'))

    def _get_pythonpath(self, error):
        return '\n'.join(self._block(error, 'PYTHONPATH:', 'CLASSPATH:'))

    def _block(self, error, start, end=None):
        include = False
        for line in str(error).splitlines():
            if line == end:
                return
            if line == start:
                include = True
            if include and line.strip('^ '):
                yield line


class TestSplitPathToModule(unittest.TestCase):

    def _verify(self, file_name, expected_name):
        path = abspath(file_name)
        actual = ByPathImporter(None)._split_path_to_module(path)
        assert_equal(actual, (dirname(path), expected_name))

    def test_normal_file(self):
        self._verify('hello.py', 'hello')
        self._verify('hello.world.pyc', 'hello.world')

    def test_directory(self):
        self._verify('hello', 'hello')
        self._verify('hello'+os.sep, 'hello')


class TestInstantiation(unittest.TestCase):

    def setUp(self):
        self.tearDown()

    def tearDown(self):
        if exists(TESTDIR):
            shutil.rmtree(TESTDIR)

    def test_when_importing_by_name(self):
        from ExampleLibrary import ExampleLibrary
        lib = Importer().import_class_or_module('ExampleLibrary',
                                                instantiate_with_args=())
        assert_true(not inspect.isclass(lib))
        assert_true(isinstance(lib, ExampleLibrary))

    def test_with_arguments(self):
        lib = Importer().import_class_or_module('libswithargs.Mixed', range(5))
        assert_equal(lib.get_args(), (0, 1, '2 3 4'))

    def test_named_arguments(self):
        lib = Importer().import_class_or_module('libswithargs.Mixed',
                                                ['default=b', 'mandatory=a'])
        assert_equal(lib.get_args(), ('a', 'b', ''))

    def test_escape_equals(self):
        lib = Importer().import_class_or_module('libswithargs.Mixed',
                                                [r'default\=b', r'mandatory\=a'])
        assert_equal(lib.get_args(), (r'default\=b', r'mandatory\=a', ''))
        lib = Importer().import_class_or_module('libswithargs.Mixed',
                                                [r'default\=b', 'default=a'])
        assert_equal(lib.get_args(), (r'default\=b', 'a', ''))

    def test_escaping_not_needed_if_args_do_not_match_names(self):
        lib = Importer().import_class_or_module('libswithargs.Mixed',
                                                ['foo=b', 'bar=a'])
        assert_equal(lib.get_args(), ('foo=b', 'bar=a', ''))

    def test_arguments_when_importing_by_path(self):
        path = create_temp_file('args.py', extra_content='''
class args:
    def __init__(self, arg='default'):
        self.arg = arg
''')
        importer = Importer().import_class_or_module_by_path
        for args, expected in [((), 'default'),
                               (['positional'], 'positional'),
                               (['arg=named'], 'named')]:
            lib = importer(path, args)
            assert_true(not inspect.isclass(lib))
            assert_equal(lib.__class__.__name__, 'args')
            assert_equal(lib.arg, expected)

    def test_instantiate_failure(self):
        assert_raises_with_msg(
            DataError,
            "Importing xxx 'ExampleLibrary' failed: Xxx 'ExampleLibrary' expected 0 arguments, got 3.",
            Importer('XXX').import_class_or_module, 'ExampleLibrary', ['accepts', 'no', 'args']
        )

    def test_argument_conversion(self):
        path = create_temp_file('conversion.py', extra_content='''
class conversion:
    def __init__(self, arg: int):
        self.arg = arg
''')
        lib = Importer().import_class_or_module_by_path(path, ['42'])
        assert_true(not inspect.isclass(lib))
        assert_equal(lib.__class__.__name__, 'conversion')
        assert_equal(lib.arg, 42)
        assert_raises_with_msg(
            DataError,
            "Importing xxx '%s' failed: "
            "Argument 'arg' got value 'invalid' that cannot be converted to integer." % path,
            Importer('XXX').import_class_or_module, path, ['invalid']
        )

    def test_modules_do_not_take_arguments(self):
        path = create_temp_file('no_args_allowed.py')
        assert_raises_with_msg(DataError,
                               "Importing '%s' failed: Modules do not take arguments." % path,
                               Importer().import_class_or_module_by_path,
                               path, ['invalid'])


if __name__ == '__main__':
    unittest.main()



from robot.api.deco import keyword, library


@library(scope='global', auto_keywords=True)
class LibraryDecoratorWithAutoKeywords:

    def undecorated_method_is_keyword(self):
        pass

    @keyword
    def decorated_method_is_keyword_as_well(self):
        pass



from ExampleLibrary import ExampleLibrary


class ExtendPythonLib(ExampleLibrary):

    def kw_in_python_extender(self, arg):
        return arg/2

    def print_many(self, *msgs):
        raise Exception('Overridden kw executed!')

    def using_method_from_python_parent(self):
        self.exception('AssertionError', 'Error message from lib')



class CustomLen:

    def __init__(self, length):
        self._length=length

    def __len__(self):
        return self._length


class LengthMethod:

    def length(self):
        return 40

    def __str__(self):
        return 'length()'


class SizeMethod:

    def size(self):
        return 41

    def __str__(self):
        return 'size()'


class LengthAttribute:
    length=42

    def __str__(self):
        return 'length'


def get_variables():
    return dict(
        CUSTOM_LEN_0=CustomLen(0),
        CUSTOM_LEN_1=CustomLen(1),
        CUSTOM_LEN_2=CustomLen(2),
        CUSTOM_LEN_3=CustomLen(3),
        LENGTH_METHOD=LengthMethod(),
        SIZE_METHOD=SizeMethod(),
        LENGTH_ATTRIBUTE=LengthAttribute()
    )



#!/usr/bin/env python

import fileinput
from os.path import join, dirname, abspath
import sys
import os

BASEDIR = dirname(abspath(__file__))
OUTPUT = join(BASEDIR, 'output.xml')

sys.path.insert(0, join(BASEDIR, '..', '..', '..', '..', 'src'))

import robot
from robot.conf.settings import RebotSettings
from robot.reporting.resultwriter import Results
from robot.reporting.jswriter import JsResultWriter, JsonWriter


def create(testdata, target, split_log=False):
    testdata = join(BASEDIR, testdata)
    output_name = target[0].lower() + target[1:-3] + 'Output'
    target = join(BASEDIR, target)
    run_robot(testdata)
    create_jsdata(target, split_log)
    inplace_replace_all(target, 'window.output', 'window.' + output_name)


def run_robot(testdata, output=OUTPUT):
    robot.run(testdata, log='NONE', report='NONE', output=output)


def create_jsdata(target, split_log, outxml=OUTPUT):
    result = Results(RebotSettings({'splitlog': split_log}), outxml).js_result
    config = {'logURL': 'log.html', 'reportURL': 'report.html', 'background': {'fail': 'DeepPink'}}
    with open(target, 'w') as output:
        JsResultWriter(output, start_block='', end_block='\n').write(result, config)
        writer = JsonWriter(output)
        for index, (keywords, strings) in enumerate(result.split_results):
            writer.write_json('window.outputKeywords%d = ' % index, keywords)
            writer.write_json('window.outputStrings%d = ' % index, strings)


def inplace_replace_all(file, search, replace):
    for line in fileinput.input(file, inplace=1):
        sys.stdout.write(line.replace(search, replace))


if __name__ == '__main__':
    create('Suite.robot', 'Suite.js')
    create('SetupsAndTeardowns.robot', 'SetupsAndTeardowns.js')
    create('Messages.robot', 'Messages.js')
    create('teardownFailure', 'TeardownFailure.js')
    create(join('teardownFailure', 'PassingFailing.robot'), 'PassingFailing.js')
    create('TestsAndKeywords.robot', 'TestsAndKeywords.js')
    create('.', 'allData.js')
    create('.', 'splitting.js', split_log=True)
    os.remove(OUTPUT)



import re
import sys
import traceback
import unittest

from robot.utils.asserts import assert_equal, assert_true, assert_raises
from robot.utils.error import get_error_details, get_error_message, ErrorDetails


def format_traceback(no_tb=False):
    e, v, tb = sys.exc_info()
    # This is needed when testing chaining and cause without traceback.
    # We set `err.__traceback__ = None` in tests and apparently that makes
    # `tb` here `None´ with Python 3.11 but not with others.
    if sys.version_info < (3, 11) and no_tb:
        tb = None
    return ''.join(traceback.format_exception(e, v, tb)).rstrip()


def format_message():
    return ''.join(traceback.format_exception_only(*sys.exc_info()[:2])).rstrip()


class TestGetErrorDetails(unittest.TestCase):

    def test_get_error_details(self):
        for exception, args, exp_msg in [
                    (AssertionError, ['My Error'], 'My Error'),
                    (AssertionError, [None], 'None'),
                    (AssertionError, [], 'AssertionError'),
                    (Exception, ['Another Error'], 'Another Error'),
                    (ValueError, ['Something'], 'ValueError: Something'),
                    (AssertionError, ['Msg\nin 3\nlines'], 'Msg\nin 3\nlines'),
                    (ValueError, ['2\nlines'], 'ValueError: 2\nlines')]:
            try:
                raise exception(*args)
            except:
                error1 = ErrorDetails()
                error2 = ErrorDetails(full_traceback=False)
                message1, tb1 = get_error_details()
                message2, tb2 = get_error_details(full_traceback=False)
                message3 = get_error_message()
                python_msg = format_message()
                python_tb = format_traceback()
            for msg in message1, message2, message3, error1.message, error2.message:
                assert_equal(msg, exp_msg)
            assert_true(tb1.startswith('Traceback (most recent call last):'))
            assert_true(tb1.endswith(exp_msg))
            assert_true(tb2.startswith('Traceback (most recent call last):'))
            assert_true(exp_msg not in tb2)
            assert_equal(tb1, error1.traceback)
            assert_equal(tb2, error2.traceback)
            assert_equal(tb1, python_tb)
            assert_equal(tb1, f'{tb2}\n{python_msg}')

    def test_chaining(self):
        try:
            1/0
        except Exception:
            try:
                raise ValueError
            except Exception:
                try:
                    raise RuntimeError('last error')
                except Exception as err:
                    assert_equal(ErrorDetails(err).traceback, format_traceback())

    def test_chaining_without_traceback(self):
        try:
            try:
                raise ValueError('lower')
            except ValueError as err:
                raise RuntimeError('higher') from err
        except Exception as err:
            err.__traceback__ = None
            assert_equal(ErrorDetails(err).traceback, format_traceback(no_tb=True))

    def test_cause(self):
        try:
            raise ValueError('err') from TypeError('cause')
        except ValueError as err:
            assert_equal(ErrorDetails(err).traceback, format_traceback())

    def test_cause_without_traceback(self):
        try:
            raise ValueError('err') from TypeError('cause')
        except ValueError as err:
            err.__traceback__ = None
            assert_equal(ErrorDetails(err).traceback, format_traceback(no_tb=True))


class TestRemoveRobotEntriesFromTraceback(unittest.TestCase):

    def test_both_robot_and_non_robot_entries(self):
        def raises():
            raise Exception
        self._verify_traceback(r'''
Traceback \(most recent call last\):
  File ".*", line \d+, in raises
    raise Exception
'''.strip(), assert_raises, AssertionError, raises)

    def test_remove_entries_with_lambda_and_multiple_entries(self):
        def raises():
            1/0
        raising_lambda = lambda: raises()
        self._verify_traceback(r'''
Traceback \(most recent call last\):
  File ".*", line \d+, in <lambda.*>
    raising_lambda = lambda: raises\(\)
  File ".*", line \d+, in raises
    1/0
'''.strip(), assert_raises, AssertionError, raising_lambda)

    def test_only_robot_entries(self):
        self._verify_traceback(r'''
Traceback \(most recent call last\):
  None
'''.strip(), assert_equal, 1, 2)

    def _verify_traceback(self, expected, method, *args):
        try:
            method(*args)
        except Exception as error:
            # first tb entry originates from this file and must be excluded
            error.__traceback__ = error.__traceback__.tb_next
            tb = ErrorDetails(error).traceback
        else:
            raise AssertionError
        # Remove lines indicating error location with `^^^^` used by Python 3.11+.
        tb = '\n'.join(line for line in tb.splitlines() if line.strip('^ '))
        if not re.match(expected, tb):
            raise AssertionError('\nExpected:\n%s\n\nActual:\n%s' % (expected, tb))


if __name__ == "__main__":
    unittest.main()



import logging
import time
import sys


class CustomHandler(logging.Handler):

    def emit(self, record):
        sys.__stdout__.write(record.getMessage().title() + '\n')


custom = logging.getLogger('custom')
custom.addHandler(CustomHandler())
nonprop = logging.getLogger('nonprop')
nonprop.propagate = False
nonprop.addHandler(CustomHandler())


class Message:

    def __init__(self, msg=''):
        self.msg = msg

    def __str__(self):
        return self.msg

    def __repr__(self):
        return repr(str(self))


class InvalidMessage(Message):

    def __str__(self):
        raise AssertionError('Should not have been logged')


def log_with_default_levels():
    logging.debug('debug message')
    logging.info('%s %s', 'info', 'message')
    logging.warning(Message('warning message'))
    logging.error('error message')
    #critical is considered a warning
    logging.critical('critical message')


def log_with_custom_levels():
    logging.log(logging.DEBUG-1, Message('below debug'))
    logging.log(logging.INFO-1, 'between debug and info')
    logging.log(logging.INFO+1, 'between info and warning')
    logging.log(logging.WARNING+5, 'between warning and error')
    logging.log(logging.ERROR*100,'above error')


def log_exception():
    try:
        raise ValueError('Bang!')
    except ValueError:
        logging.exception('Error occurred!')


def log_invalid_message():
    logging.info(InvalidMessage())


def log_using_custom_logger():
    logging.getLogger('custom').info('custom logger')


def log_using_non_propagating_logger():
    logging.getLogger('nonprop').info('nonprop logger')


def log_messages_different_time():
    logging.info('First message')
    time.sleep(0.1)
    logging.info('Second message 0.1 sec later')


def log_with_format():
    test_format = "%(name)s %(levelname)s %(message)s"
    formatter = logging.Formatter(fmt=test_format)
    root = logging.getLogger()
    handler = root.handlers[0]
    old_formatter = handler.formatter
    handler.setFormatter(formatter)
    logging.info("logged at info")
    handler.setFormatter(old_formatter)


def log_something():
    logging.info('something')



import os
import unittest
import pathlib

from robot.utils.asserts import assert_equal, assert_true
from robot.utils.etreewrapper import ETSource, ET


PATH = os.path.join(os.path.dirname(__file__), 'test_etreesource.py')


class TestETSource(unittest.TestCase):

    def test_path(self):
        self._test_path(PATH)

    def _test_path(self, path, string_repr=None, expected=None):
        source = ETSource(path)
        with source as src:
            assert_equal(src, expected or path)
        self._verify_string_representation(source, string_repr or path)
        assert_true(source._opened is None)

    def test_bytes_path(self):
        self._test_path(os.fsencode(PATH), PATH)

    def test_pathlib_path(self):
        self._test_path(pathlib.Path(PATH), PATH, pathlib.Path(PATH))

    def test_opened_file_object(self):
        with open(PATH) as f:
            source = ETSource(f)
            with source as src:
                assert_true(src.read().startswith('import os'))
                assert_true(src is f)
            assert_true(src.closed is False)
            self._verify_string_representation(source, PATH)
            assert_true(source._opened is None)
            assert_true(src is f)
        assert_true(src.closed is True)

    def test_string(self):
        self._test_string('\n<tag>content</tag>\n')

    def test_byte_string(self):
        self._test_string(b'\n<tag>content</tag>')
        self._test_string('<tag>hyv\xe4</tag>'.encode('utf8'))
        self._test_string('<?xml version="1.0" encoding="Latin1"?>\n'
                          '<tag>hyv\xe4</tag>'.encode('latin-1'), 'latin-1')

    def test_unicode_string(self):
        self._test_string('\n<tag>hyv\xe4</tag>\n')
        self._test_string('<?xml version="1.0" encoding="latin1"?>\n'
                          '<tag>hyv\xe4</tag>', 'latin-1')
        self._test_string("<?xml version='1.0' encoding='iso-8859-1' standalone='yes'?>\n"
                          "<tag>hyv\xe4</tag>", 'latin-1')

    def _test_string(self, xml, encoding='UTF-8'):
        source = ETSource(xml)
        with source as src:
            content = src.read()
            expected = xml if isinstance(xml, bytes) else xml.encode(encoding)
            assert_equal(content, expected)
        self._verify_string_representation(source, '<in-memory file>')
        assert_true(source._opened.closed)
        with ETSource(xml) as src:
            assert_equal(ET.parse(src).getroot().tag, 'tag')

    def test_non_ascii_string_repr(self):
        self._verify_string_representation(ETSource('\xe4'), '\xe4')

    def _verify_string_representation(self, source, expected):
        assert_equal(str(source), expected)
        assert_equal('-%s-' % source, '-%s-' % expected)


if __name__ == '__main__':
    unittest.main()



class RecLibrary1:

    def keyword_only_in_library_1(self):
        print("Keyword from library 1")

    def keyword_in_both_libraries(self):
        print("Keyword from library 1")

    def keyword_in_all_resources_and_libraries(self):
        print("Keyword from library 1")

    def keyword_everywhere(self):
        print("Keyword from library 1")

    def no_operation(self):
        print("Overrides keyword from BuiltIn library")

    def similar_kw_3(self):
        pass

    def action(self):
        pass

    def do_action(self):
        pass

    def action_and_expect_problems(self):
        pass

    def action_and_ignore_problems(self):
        pass

    def wait_until_action_succeeds(self):
        pass

    def do_stuff(self):
        pass

    def open_application(self):
        pass

    def ask_user_for_input(self):
        pass

    def boot_up_server(self):
        pass

    def shut_down_server(self):
        pass

    def minimize_window(self):
        pass

    def maximize_window(self):
        pass

    def open_window(self):
        pass

    def create_data(self):
        pass

    def delete_data(self):
        pass

    def update_data(self):
        pass

    def modify_data(self):
        pass

    def get_data(self):
        pass

    def read_data(self):
        pass

    def record(self, message, level):
        pass

    def nothing(self):
        pass



class DeprecatedKeywords:

    def deprecated_library_keyword(self):
        """*DEPRECATED* Use keyword `Not Deprecated With Doc` instead!

        Some more doc here. Ignore this in the warning.
        """
        pass

    def deprecated_library_keyword_with_multiline_message(self):
        """*DEPRECATED* Multiline
        message.

        Some more doc here. Ignore this in the warning.
        """
        pass

    def deprecated_library_keyword_without_extra_doc(self):
        """*DEPRECATED*"""
        pass

    def deprecated_library_keyword_with_stuff_to_ignore(self):
        """*DEPRECATED ignore this stuff*"""
        pass

    def deprecated_keyword_returning(self):
        """*DEPRECATED.* But still returning a value!"""
        return 42

    def not_deprecated_with_doc(self):
        """Some Short Doc

        Some more doc and ignore this *DEPRECATED*
        """
        pass

    def not_deprecated_with_deprecated_prefix(self):
        """*DEPRECATED ... just kidding!!"""
        pass

    def not_deprecated_without_doc(self):
        pass



import os
import time


class MyException(Exception):
    pass


def passing(*args):
    pass


def sleeping(s):
    seconds = s
    while seconds > 0:
        time.sleep(min(seconds, 0.1))
        seconds -= 0.1
    os.environ['ROBOT_THREAD_TESTING'] = str(s)
    return s


def returning(arg):
    return arg


def failing(msg='xxx'):
    raise MyException(msg)



__all__ = ['VAR']


class Demeter:
    loves = ''
    @property
    def hates(self):
        return self.loves.upper()


class Variable:
    attr = 'value'
    _attr2 = 'v2'
    attr2 = property(lambda self: self._attr2,
                     lambda self, value: setattr(self, '_attr2', value.upper()))
    demeter = Demeter()
    @property
    def not_settable(self):
        return None


VAR = Variable()



import json
import unittest
from io import StringIO

from robot.utils.asserts import assert_equal, assert_raises
from robot.htmldata.jsonwriter import JsonDumper


class TestJsonDumper(unittest.TestCase):

    def _dump(self, data):
        output = StringIO()
        JsonDumper(output).dump(data)
        return output.getvalue()

    def _test(self, data, expected):
        assert_equal(self._dump(data), expected)

    def test_dump_string(self):
        self._test('', '""')
        self._test('hello world', '"hello world"')
        self._test('123', '"123"')

    def test_dump_non_ascii_string(self):
        self._test(u'hyv\xe4', u'"hyv\xe4"')

    def test_escape_string(self):
        self._test('"-\\-\n-\t-\r', '"\\"-\\\\-\\n-\\t-\\r"')

    def test_escape_closing_tags(self):
        self._test('<script><></script>', '"<script><>\\x3c/script>"')

    def test_dump_boolean(self):
        self._test(True, 'true')
        self._test(False, 'false')

    def test_dump_integer(self):
        self._test(12, '12')
        self._test(-12312, '-12312')
        self._test(0, '0')
        self._test(1, '1')

    def test_dump_long(self):
        self._test(12345678901234567890, '12345678901234567890')

    def test_dump_list(self):
        self._test([1, 2, True, 'hello', 'world'], '[1,2,true,"hello","world"]')
        self._test(['*nes"ted', [1, 2, [4]]], '["*nes\\"ted",[1,2,[4]]]')

    def test_dump_tuple(self):
        self._test(('hello', '*world'), '["hello","*world"]')
        self._test((1, 2, (3, 4)), '[1,2,[3,4]]')

    def test_dump_dictionary(self):
        self._test({'key': 1}, '{"key":1}')
        self._test({'nested': [-1, {42: None}]}, '{"nested":[-1,{42:null}]}')

    def test_dictionaries_are_sorted(self):
        self._test({'key': 1, 'hello': ['wor', 'ld'], 'z': 'a', 'a': 'z'},
                   '{"a":"z","hello":["wor","ld"],"key":1,"z":"a"}')

    def test_dump_none(self):
        self._test(None, 'null')

    def test_json_dump_mapping(self):
        output = StringIO()
        dumper = JsonDumper(output)
        mapped1 = object()
        mapped2 = 'string'
        dumper.dump([mapped1, [mapped2, {mapped2: mapped1}]],
                    mapping={mapped1: '1', mapped2: 'a'})
        assert_equal(output.getvalue(), '[1,[a,{a:1}]]')
        assert_raises(ValueError, dumper.dump, [mapped1])

    def test_against_standard_json(self):
        data = ['\\\'\"\r\t\n' + ''.join(chr(i) for i in range(32, 127)),
                {'A': 1, 'b': 2, 'C': ()}, None, (1, 2, 3)]
        expected = json.dumps(data, sort_keys=True, separators=(',', ':'))
        self._test(data, expected)


if __name__ == '__main__':
    unittest.main()



import sys
import os

from robot.api import SuiteVisitor


ROBOT_LISTENER_API_VERSION = 3


def start_suite(data, result):
    data.name = data.doc = result.name = 'Not visible in results'
    result.doc = (result.doc + ' [start suite]').strip()
    result.metadata['suite'] = '[start]'
    result.metadata['tests'] = ''
    result.metadata['number'] = 42
    assert len(data.tests) == 2
    assert len(result.tests) == 0
    data.tests.create(name='Added by start_suite')
    data.visit(TestModifier())


def end_suite(data, result):
    assert len(data.tests) == 5, '%d tests, not 5' % len(data.tests)
    assert len(result.tests) == 5, '%d tests, not 5' % len(result.tests)
    assert data.name == data.doc == result.name == 'Not visible in results'
    assert result.doc.endswith('[start suite]')
    assert result.metadata['suite'] == '[start]'
    assert result.metadata['tests'] == 'xxxxx'
    assert result.metadata['number'] == '42'
    result.name += ' [end suite]'
    result.doc += ' [end suite]'
    result.metadata['suite'] += ' [end]'
    for test in result.tests:
        test.name = 'Not visible in reports'
        test.status = 'PASS'    # Not visible in reports


def startTest(data, result):
    data.name = data.doc = result.name = 'Not visible in results'
    result.doc = (result.doc + ' [start test]').strip()
    result.tags.add('[start]')
    result.message = '[start]'
    result.parent.metadata['tests'] += 'x'
    data.body.create_keyword('No Operation')
    if data is data.parent.tests[-1] and 'dynamic' not in data.tags:
        new = data.parent.tests.create(name='Added by startTest',
                                       tags=['dynamic', 'start'])
        new.body.create_keyword(name='Fail', args=['Dynamically added!'])


def end_test(data, result):
    result.name = 'Does not go to output.xml'
    result.doc += ' [end test]'
    result.tags.add('[end]')
    result.passed = not result.passed
    result.message += ' [end]'
    if 'dynamic' in data.tags and 'start' in data.tags:
        new = data.parent.tests.create(name='Added by end_test',
                                       doc='Dynamic',
                                       tags=['dynamic', 'end'])
        new.body.create_keyword(name='Log', args=['Dynamically added!', 'INFO'])
    data.name = data.doc = 'Not visible in results'


def log_message(msg):
    msg.message = msg.message.upper()
    msg.timestamp = '20151216 15:51:20.141'


message = log_message


def output_file(path):
    print("Output: %s" % os.path.basename(path), file=sys.__stderr__)


def log_file(path):
    print("Log: %s" % os.path.basename(path), file=sys.__stderr__)


def report_file(path):
    print("Report: %s" % os.path.basename(path), file=sys.__stderr__)


def debug_file(path):
    print("Debug: %s" % os.path.basename(path), file=sys.__stderr__)


def xunit_file(path):
    print("Xunit: %s" % os.path.basename(path), file=sys.__stderr__)


def close():
    print("Close", file=sys.__stderr__)


class TestModifier(SuiteVisitor):

    def visit_test(self, test):
        test.name += ' [start suite]'
        test.doc = (test.doc + ' [start suite]').strip()
        test.tags.add('[start suite]')


def not_implemented(*args):
    raise SystemExit('Should not be called!')


start_keyword = end_keyword = not_implemented
library_import = resource_import = variables_import = not_implemented



sp = ' '
hash = '#'
bs = '\\'
tab = '\t'
nl = '\n'
cr = '\r'
x00 = '\x00'
xE4 = '\xE4'
xFF = '\xFF'
u2603 = '\u2603'  # SNOWMAN
uFFFF = '\uFFFF'
U00010905 = '\U00010905'  # PHOENICIAN LETTER WAU
U0010FFFF = '\U0010FFFF'  # Biggest valid Unicode character
var = '${non_existing}'
pipe = '|'



from os.path import abspath, dirname, join, normpath
import locale
import os
import subprocess

import robot


__all__ = ['ROBOTPATH', 'ROBOT_VERSION', 'DATADIR', 'SYSTEM_ENCODING',
           'CONSOLE_ENCODING']


ROBOTPATH = dirname(abspath(robot.__file__))
ROBOT_VERSION = robot.version.get_version()
DATADIR = normpath(join(dirname(abspath(__file__)), '..', 'testdata'))

SYSTEM_ENCODING = locale.getpreferredencoding(False)
# Python 3.6+ uses UTF-8 internally on Windows. We want real console encoding.
if os.name == 'nt':
    output = subprocess.check_output('chcp', shell=True, encoding='ASCII',
                                     errors='ignore')
    CONSOLE_ENCODING = 'cp' + output.split()[-1]
else:
    CONSOLE_ENCODING = locale.getdefaultlocale()[-1]



import unittest

from robot.utils.asserts import assert_equal, assert_raises_with_msg
from robot.libraries.BuiltIn import BuiltIn, RobotNotRunningError
from robot.libraries.DateTime import Date


class TestBuiltInWhenRobotNotRunning(unittest.TestCase):

    def test_using_namespace(self):
        assert_raises_with_msg(RobotNotRunningError,
                               'Cannot access execution context',
                               BuiltIn().get_variables)

    def test_using_namespace_backwards_compatibility(self):
        assert_raises_with_msg(AttributeError,
                               'Cannot access execution context',
                               BuiltIn().get_variables)

    def test_suite_doc_and_metadata(self):
        assert_raises_with_msg(RobotNotRunningError,
                               'Cannot access execution context',
                               BuiltIn().set_suite_documentation, 'value')
        assert_raises_with_msg(RobotNotRunningError,
                               'Cannot access execution context',
                               BuiltIn().set_suite_metadata, 'name', 'value')


class TestBuiltInPropertys(unittest.TestCase):

    def test_robot_running(self):
        assert_equal(BuiltIn().robot_running, False)

    def test_dry_run_active(self):
        assert_equal(BuiltIn().dry_run_active, False)


class TestDateTime(unittest.TestCase):

    def test_date_seconds(self):
        secs = 1234567890
        assert_equal(Date(secs).seconds, secs)


if __name__ == '__main__':
    unittest.main()



class DictWithoutHasKey(dict):

    def has_key(self, key):
        raise NotImplementedError('Emulating collections.Mapping which '
                                  'does not have `has_key`.')


def get_dict_without_has_key(**items):
    return DictWithoutHasKey(**items)



class NamedArgsImportLibrary:

    def __init__(self, arg1=None, arg2=None, **kws):
        self.arg1 = arg1
        self.arg2 = arg2
        self.kws = kws

    def check_init_arguments(self, exp_arg1, exp_arg2, **kws):
        if self.arg1 != exp_arg1 or self.arg2 != exp_arg2 or kws != self.kws:
            raise AssertionError('Wrong initialization values. Got (%s, %s, %r), expected (%s, %s, %r)'
                                 % (self.arg1, self.arg2, self.kws, exp_arg1, exp_arg2, kws))



ROBOT_LISTENER_API_VERSION = 3


def startTest(data, result):
    result.message = '[START] [original] %s [resolved] %s' % (data.name, result.name)


def end_test(data, result):
    result.message += '\n[END] [original] %s [resolved] %s' % (data.name, result.name)



import logging
import signal
import sys
import unittest
from io import StringIO
from os.path import abspath, dirname, join

from robot.running import TestSuite, TestSuiteBuilder
from robot.utils.asserts import assert_equal

from resources.runningtestcase import RunningTestCase
from resources.Listener import Listener


CURDIR = dirname(abspath(__file__))
ROOTDIR = dirname(dirname(CURDIR))
DATADIR = join(ROOTDIR, 'atest', 'testdata', 'misc')


def run(suite, **kwargs):
    config = dict(output=None, log=None, report=None,
                  stdout=StringIO(), stderr=StringIO())
    config.update(kwargs)
    result = suite.run(**config)
    return result.suite


def build(path):
    return TestSuiteBuilder().build(join(DATADIR, path))


def assert_suite(suite, name, status, message='', tests=1):
    assert_equal(suite.name, name)
    assert_equal(suite.status, status)
    assert_equal(suite.message, message)
    assert_equal(len(suite.tests), tests)


def assert_test(test, name, status, tags=(), msg=''):
    assert_equal(test.name, name)
    assert_equal(test.status, status)
    assert_equal(test.message, msg)
    assert_equal(tuple(test.tags), tags)


def assert_signal_handler_equal(signum, expected):
    sig = signal.getsignal(signum)
    assert_equal(sig, expected)


class TestRunning(unittest.TestCase):

    def test_one_library_keyword(self):
        suite = TestSuite(name='Suite')
        suite.tests.create(name='Test').body.create_keyword('Log', args=['Hello!'])
        result = run(suite)
        assert_suite(result, 'Suite', 'PASS')
        assert_test(result.tests[0], 'Test', 'PASS')

    def test_failing_library_keyword(self):
        suite = TestSuite(name='Suite')
        test = suite.tests.create(name='Test')
        test.body.create_keyword('Log', args=['Dont fail yet.'])
        test.body.create_keyword('Fail', args=['Hello, world!'])
        result = run(suite)
        assert_suite(result, 'Suite', 'FAIL')
        assert_test(result.tests[0], 'Test', 'FAIL', msg='Hello, world!')

    def test_assign(self):
        suite = TestSuite(name='Suite')
        test = suite.tests.create(name='Test')
        test.body.create_keyword(assign=['${var}'], name='Set Variable',
                                 args=['value in variable'])
        test.body.create_keyword('Fail', args=['${var}'])
        result = run(suite)
        assert_suite(result, 'Suite', 'FAIL')
        assert_test(result.tests[0], 'Test', 'FAIL', msg='value in variable')

    def test_suites_in_suites(self):
        root = TestSuite(name='Root')
        root.suites.create(name='Child')\
            .tests.create(name='Test')\
            .body.create_keyword('Log', args=['Hello, world!'])
        result = run(root)
        assert_suite(result, 'Root', 'PASS', tests=0)
        assert_suite(result.suites[0], 'Child', 'PASS')
        assert_test(result.suites[0].tests[0], 'Test', 'PASS')

    def test_user_keywords(self):
        suite = TestSuite(name='Suite')
        suite.tests.create(name='Test')\
            .body.create_keyword('User keyword', args=['From uk'])
        uk = suite.resource.keywords.create(name='User keyword', args=['${msg}'])
        uk.body.create_keyword(name='Fail', args=['${msg}'])
        result = run(suite)
        assert_suite(result, 'Suite', 'FAIL')
        assert_test(result.tests[0], 'Test', 'FAIL', msg='From uk')

    def test_variables(self):
        suite = TestSuite(name='Suite')
        suite.resource.variables.create('${ERROR}', ['Error message'])
        suite.resource.variables.create('@{LIST}', ['Error', 'added tag'])
        suite.tests.create(name='T1').body.create_keyword('Fail', args=['${ERROR}'])
        suite.tests.create(name='T2').body.create_keyword('Fail', args=['@{LIST}'])
        result = run(suite)
        assert_suite(result, 'Suite', 'FAIL', tests=2)
        assert_test(result.tests[0], 'T1', 'FAIL', msg='Error message')
        assert_test(result.tests[1], 'T2', 'FAIL', ('added tag',), 'Error')

    def test_test_cannot_be_empty(self):
        suite = TestSuite()
        suite.tests.create(name='Empty')
        result = run(suite)
        assert_test(result.tests[0], 'Empty', 'FAIL', msg='Test cannot be empty.')

    def test_name_cannot_be_empty(self):
        suite = TestSuite()
        suite.tests.create().body.create_keyword('Not executed')
        result = run(suite)
        assert_test(result.tests[0], '', 'FAIL', msg='Test name cannot be empty.')

    def test_modifiers_are_not_used(self):
        # These options are valid but not used. Modifiers can be passed to
        # suite.visit() explicitly if needed.
        suite = TestSuite(name='Suite')
        suite.tests.create(name='Test').body.create_keyword('No Operation')
        result = run(suite, prerunmodifier='not used', prerebotmodifier=42)
        assert_suite(result, 'Suite', 'PASS', tests=1)


class TestTestSetupAndTeardown(unittest.TestCase):

    def setUp(self):
        self.tests = run(build('setups_and_teardowns.robot')).tests

    def test_passing_setup_and_teardown(self):
        assert_test(self.tests[0], 'Test with setup and teardown', 'PASS')

    def test_failing_setup(self):
        assert_test(self.tests[1], 'Test with failing setup', 'FAIL',
                    msg='Setup failed:\nTest Setup')

    def test_failing_teardown(self):
        assert_test(self.tests[2], 'Test with failing teardown', 'FAIL',
                    msg='Teardown failed:\nTest Teardown')

    def test_failing_test_with_failing_teardown(self):
        assert_test(self.tests[3], 'Failing test with failing teardown', 'FAIL',
                    msg='Keyword\n\nAlso teardown failed:\nTest Teardown')


class TestSuiteSetupAndTeardown(unittest.TestCase):

    def setUp(self):
        self.suite = build('setups_and_teardowns.robot')

    def test_passing_setup_and_teardown(self):
        suite = run(self.suite)
        assert_suite(suite, 'Setups And Teardowns', 'FAIL', tests=4)
        assert_test(suite.tests[0], 'Test with setup and teardown', 'PASS')

    def test_failing_setup(self):
        suite = run(self.suite, variable='SUITE SETUP:Fail')
        assert_suite(suite, 'Setups And Teardowns', 'FAIL',
                     'Suite setup failed:\nAssertionError', 4)
        assert_test(suite.tests[0], 'Test with setup and teardown', 'FAIL',
                    msg='Parent suite setup failed:\nAssertionError')

    def test_failing_teardown(self):
        suite = run(self.suite, variable='SUITE TEARDOWN:Fail')
        assert_suite(suite, 'Setups And Teardowns', 'FAIL',
                     'Suite teardown failed:\nAssertionError', 4)
        assert_test(suite.tests[0], 'Test with setup and teardown', 'FAIL',
                    msg='Parent suite teardown failed:\nAssertionError')

    def test_failing_test_with_failing_teardown(self):
        suite = run(self.suite, variable=['SUITE SETUP:Fail', 'SUITE TEARDOWN:Fail'])
        assert_suite(suite, 'Setups And Teardowns', 'FAIL',
                     'Suite setup failed:\nAssertionError\n\n'
                     'Also suite teardown failed:\nAssertionError', 4)
        assert_test(suite.tests[0], 'Test with setup and teardown', 'FAIL',
                    msg='Parent suite setup failed:\nAssertionError\n\n'
                        'Also parent suite teardown failed:\nAssertionError')

    def test_nested_setups_and_teardowns(self):
        root = TestSuite(name='Root')
        root.teardown.config(name='Fail', args=['Top level'], type='teardown')
        root.suites.append(self.suite)
        suite = run(root, variable=['SUITE SETUP:Fail', 'SUITE TEARDOWN:Fail'])
        assert_suite(suite, 'Root', 'FAIL',
                     'Suite teardown failed:\nTop level', 0)
        assert_suite(suite.suites[0], 'Setups And Teardowns', 'FAIL',
                     'Suite setup failed:\nAssertionError\n\n'
                     'Also suite teardown failed:\nAssertionError', 4)
        assert_test(suite.suites[0].tests[0], 'Test with setup and teardown', 'FAIL',
                    msg='Parent suite setup failed:\nAssertionError\n\n'
                        'Also parent suite teardown failed:\nAssertionError\n\n'
                        'Also parent suite teardown failed:\nTop level')


class TestCustomStreams(RunningTestCase):

    def test_stdout_and_stderr(self):
        self._run()
        self._assert_output(sys.__stdout__,
                            [('My Suite', 2), ('My Test', 1),
                             ('1 test, 1 passed, 0 failed', 1)])
        self._assert_output(sys.__stderr__, [('Hello, world!', 1)])

    def test_custom_stdout_and_stderr(self):
        stdout, stderr = StringIO(), StringIO()
        self._run(stdout, stderr)
        self._assert_normal_stdout_stderr_are_empty()
        self._assert_output(stdout, [('My Suite', 2), ('My Test', 1)])
        self._assert_output(stderr, [('Hello, world!', 1)])

    def test_same_custom_stdout_and_stderr(self):
        output = StringIO()
        self._run(output, output)
        self._assert_normal_stdout_stderr_are_empty()
        self._assert_output(output, [('My Suite', 2), ('My Test', 1),
                                     ('Hello, world!', 1)])

    def test_run_multiple_times_with_different_stdout_and_stderr(self):
        stdout, stderr = StringIO(), StringIO()
        self._run(stdout, stderr)
        self._assert_normal_stdout_stderr_are_empty()
        self._assert_output(stdout, [('My Suite', 2), ('My Test', 1)])
        self._assert_output(stderr, [('Hello, world!', 1)])
        stdout.close(); stderr.close()
        output = StringIO()
        self._run(output, output, variable='MESSAGE:Hi, again!')
        self._assert_normal_stdout_stderr_are_empty()
        self._assert_output(output, [('My Suite', 2), ('My Test', 1),
                                     ('Hi, again!', 1), ('Hello, world!', 0)])
        output.close()
        self._run(variable='MESSAGE:Last hi!')
        self._assert_output(sys.__stdout__, [('My Suite', 2), ('My Test', 1)])
        self._assert_output(sys.__stderr__, [('Last hi!', 1), ('Hello, world!', 0)])

    def _run(self, stdout=None, stderr=None, **options):
        suite = TestSuite(name='My Suite')
        suite.resource.variables.create('${MESSAGE}', ['Hello, world!'])
        suite.tests.create(name='My Test')\
            .body.create_keyword('Log', args=['${MESSAGE}', 'WARN'])
        run(suite, stdout=stdout, stderr=stderr, **options)

    def _assert_normal_stdout_stderr_are_empty(self):
        self._assert_outputs()


class TestPreservingSignalHandlers(unittest.TestCase):

    def setUp(self):
        self.orig_sigint = signal.getsignal(signal.SIGINT)
        self.orig_sigterm = signal.getsignal(signal.SIGTERM)

    def tearDown(self):
        signal.signal(signal.SIGINT, self.orig_sigint)
        signal.signal(signal.SIGTERM, self.orig_sigterm)

    def test_original_signal_handlers_are_restored(self):
        my_sigterm = lambda signum, frame: None
        signal.signal(signal.SIGTERM, my_sigterm)
        suite = TestSuite(name='My Suite')
        suite.tests.create(name='My Test').body.create_keyword('Log', args=['Hi!'])
        run(suite)
        assert_signal_handler_equal(signal.SIGINT, self.orig_sigint)
        assert_signal_handler_equal(signal.SIGTERM, my_sigterm)


class TestStateBetweenTestRuns(unittest.TestCase):

    def test_reset_logging_conf(self):
        assert_equal(logging.getLogger().handlers, [])
        assert_equal(logging.raiseExceptions, 1)
        suite = TestSuite(name='My Suite')
        suite.tests.create(name='My Test').body.create_keyword('Log', args=['Hi!'])
        run(suite)
        assert_equal(logging.getLogger().handlers, [])
        assert_equal(logging.raiseExceptions, 1)


class TestListeners(RunningTestCase):

    def test_listeners(self):
        module_file = join(ROOTDIR, 'utest', 'resources', 'Listener.py')
        suite = build('setups_and_teardowns.robot')
        suite.run(output=None, log=None, report=None, listener=[module_file+":1", Listener(2)])
        self._assert_outputs([("[from listener 1]", 1), ("[from listener 2]", 1)])

    def test_listeners_unregistration(self):
        module_file = join(ROOTDIR, 'utest', 'resources', 'Listener.py')
        suite = build('setups_and_teardowns.robot')
        suite.run(output=None, log=None, report=None, listener=module_file+":1")
        self._assert_outputs([("[from listener 1]", 1), ("[listener close]", 1)])
        self._clear_outputs()
        suite.run(output=None, log=None, report=None)
        self._assert_outputs([("[from listener 1]", 0), ("[listener close]", 0)])


if __name__ == '__main__':
    unittest.main()



import sys


def close():
    sys.exit('This should not be called')


class V1ClassListener:
    ROBOT_LISTENER_API_VERSION = 1

    def close(self):
        close()


class InvalidVersionClassListener:
    ROBOT_LISTENER_API_VERSION = 'kekkonen'

    def close(self):
        close()



import unittest
from collections import UserDict

from robot.utils import normalize, NormalizedDict
from robot.utils.asserts import assert_equal, assert_true, assert_false, assert_raises


class TestNormalize(unittest.TestCase):

    def _verify(self, string, expected, **config):
        assert_equal(normalize(string, **config), expected)

    def test_defaults(self):
        for inp, exp in [('', ''),
                         ('            ', ''),
                         (' \n\t\r', ''),
                         ('foo', 'foo'),
                         ('BAR', 'bar'),
                         (' f o o ', 'foo'),
                         ('_BAR', '_bar'),
                         ('Fo OBar\r\n', 'foobar'),
                         ('foo\tbar', 'foobar'),
                         ('\n \n \n \n F o O \t\tBaR \r \r \r   ', 'foobar')]:
            self._verify(inp, exp)

    def test_caseless(self):
        self._verify('Fo o BaR', 'FooBaR', caseless=False)
        self._verify('Fo o BaR', 'foobar', caseless=True)

    def test_caseless_non_ascii(self):
        self._verify('\xc4iti', '\xc4iti', caseless=False)
        for mother in ['\xc4ITI', '\xc4iTi', '\xe4iti', '\xe4iTi']:
            self._verify(mother, '\xe4iti', caseless=True)

    def test_spaceless(self):
        self._verify('Fo o BaR', 'fo o bar', spaceless=False)
        self._verify('Fo o BaR', 'foobar', spaceless=True)

    def test_ignore(self):
        self._verify('Foo_ bar', 'fbar', ignore=['_', 'x', 'o'])
        self._verify('Foo_ bar', 'fbar', ignore=('_', 'x', 'o'))
        self._verify('Foo_ bar', 'fbar', ignore='_xo')
        self._verify('Foo_ bar', 'bar', ignore=['_', 'f', 'o'])
        self._verify('Foo_ bar', 'bar', ignore=['_', 'F', 'O'])
        self._verify('Foo_ bar', 'Fbar', ignore=['_', 'f', 'o'], caseless=False)
        self._verify('Foo_\n bar\n', 'foo_ bar', ignore=['\n'], spaceless=False)

    def test_bytes(self):
        self._verify(b'FOO & Bar', b'foo&bar')
        self._verify(b'FOO & Bar', b'oobar', ignore=[b'F', b'&'])
        self._verify(b'FOO & Bar', b'oo  bar', ignore=[b'F', b'&'], spaceless=False)
        self._verify(b'FOO & Bar', b'bar', ignore=b'F&o')
        self._verify(b'FOO & Bar', b'OOBar', ignore=b'F&o', caseless=False)

    def test_string_subclass_without_compatible_init(self):
        class BrokenLikeSudsText(str):
            def __new__(cls, value):
                return str.__new__(cls, value)
        self._verify(BrokenLikeSudsText('suds.sax.text.Text is BROKEN'),
                     'suds.sax.text.textisbroken')
        self._verify(BrokenLikeSudsText(''), '')


class TestNormalizedDict(unittest.TestCase):

    def test_default_constructor(self):
        nd = NormalizedDict()
        nd['foo bar'] = 'value'
        assert_equal(nd['foobar'], 'value')
        assert_equal(nd['F  oo\nBar'], 'value')

    def test_initial_values_as_dict(self):
        nd = NormalizedDict({'key': 'value', 'F O\tO': 'bar'})
        assert_equal(nd['key'], 'value')
        assert_equal(nd['K EY'], 'value')
        assert_equal(nd['foo'], 'bar')

    def test_initial_values_as_name_value_pairs(self):
        nd = NormalizedDict([('key', 'value'), ('F O\tO', 'bar')])
        assert_equal(nd['key'], 'value')
        assert_equal(nd['K EY'], 'value')
        assert_equal(nd['foo'], 'bar')

    def test_initial_values_as_generator(self):
        nd = NormalizedDict((item for item in [('key', 'value'), ('F O\tO', 'bar')]))
        assert_equal(nd['key'], 'value')
        assert_equal(nd['K EY'], 'value')
        assert_equal(nd['foo'], 'bar')

    def test_setdefault(self):
        nd = NormalizedDict({'a': NormalizedDict()})
        nd.setdefault('a').setdefault('B', []).append(1)
        nd.setdefault('A', 'whatever').setdefault('b', []).append(2)
        assert_equal(nd['a']['b'], [1, 2])
        assert_equal(list(nd), ['a'])
        assert_equal(list(nd['a']), ['B'])

    def test_ignore(self):
        nd = NormalizedDict(ignore=['_'])
        nd['foo_bar'] = 'value'
        assert_equal(nd['foobar'], 'value')
        assert_equal(nd['F  oo\nB   ___a r'], 'value')

    def test_caseless_and_spaceless(self):
        nd1 = NormalizedDict({'F o o BAR': 'value'})
        nd2 = NormalizedDict({'F o o BAR': 'value'}, caseless=False,
                             spaceless=False)
        assert_equal(nd1['F o o BAR'], 'value')
        assert_equal(nd2['F o o BAR'], 'value')
        nd1['FooBAR'] = 'value 2'
        nd2['FooBAR'] = 'value 2'
        assert_equal(nd1['F o o BAR'], 'value 2')
        assert_equal(nd2['F o o BAR'], 'value')
        assert_equal(nd1['FooBAR'], 'value 2')
        assert_equal(nd2['FooBAR'], 'value 2')
        for key in ['foobar', 'f o o b ar', 'Foo BAR']:
            assert_equal(nd1[key], 'value 2')
            assert_raises(KeyError, nd2.__getitem__, key)
            assert_true(key not in nd2)

    def test_caseless_with_non_ascii(self):
        nd1 = NormalizedDict({'\xe4': 1})
        assert_equal(nd1['\xe4'], 1)
        assert_equal(nd1['\xc4'], 1)
        assert_true('\xc4' in nd1)
        nd2 = NormalizedDict({'\xe4': 1}, caseless=False)
        assert_equal(nd2['\xe4'], 1)
        assert_true('\xc4' not in nd2)

    def test_contains(self):
        nd = NormalizedDict({'Foo': 'bar'})
        assert_true('Foo' in nd and 'foo' in nd and 'FOO' in nd)

    def test_original_keys_are_preserved(self):
        nd = NormalizedDict({'low': 1, 'UP': 2})
        nd['up'] = nd['Spa Ce'] = 3
        assert_equal(list(nd.keys()), ['low', 'Spa Ce', 'UP'])
        assert_equal(list(nd.items()), [('low', 1), ('Spa Ce', 3), ('UP', 3)])

    def test_deleting_items(self):
        nd = NormalizedDict({'A': 1, 'b': 2})
        del nd['A']
        del nd['B']
        assert_equal(nd._data, {})
        assert_equal(list(nd.keys()), [])

    def test_pop(self):
        nd = NormalizedDict({'A': 1, 'b': 2})
        assert_equal(nd.pop('A'), 1)
        assert_equal(nd.pop('B'), 2)
        assert_equal(nd._data, {})
        assert_equal(list(nd.keys()), [])

    def test_pop_with_default(self):
        assert_equal(NormalizedDict().pop('nonex', 'default'), 'default')

    def test_popitem(self):
        items = [(str(i), i) for i in range(9)]
        nd = NormalizedDict(items)
        for i in range(9):
            assert_equal(nd.popitem(), items[i])
        assert_equal(nd._data, {})
        assert_equal(list(nd.keys()), [])

    def test_popitem_empty(self):
        assert_raises(KeyError, NormalizedDict().popitem)

    def test_len(self):
        nd = NormalizedDict()
        assert_equal(len(nd), 0)
        nd['a'] = nd['b'] = nd['B'] = nd['c'] = 'x'
        assert_equal(len(nd), 3)

    def test_truth_value(self):
        assert_false(NormalizedDict())
        assert_true(NormalizedDict({'a': 1}))

    def test_copy(self):
        nd = NormalizedDict({'a': 1, 'B': 1})
        cd = nd.copy()
        assert_equal(nd, cd)
        assert_equal(nd._data, cd._data)
        assert_equal(nd._keys, cd._keys)
        assert_equal(nd._normalize, cd._normalize)
        nd['C'] = 1
        cd['b'] = 2
        assert_equal(nd._keys, {'a': 'a', 'b': 'B', 'c': 'C'})
        assert_equal(nd._data, {'a': 1, 'b': 1, 'c': 1})
        assert_equal(cd._keys, {'a': 'a', 'b': 'B'})
        assert_equal(cd._data, {'a': 1, 'b': 2})

    def test_copy_with_subclass(self):
        class SubClass(NormalizedDict):
            pass
        assert_true(isinstance(SubClass().copy(), SubClass))

    def test_str(self):
        nd = NormalizedDict({'a': 1, 'B': 2, 'c': '3', 'd': '"', 'E': 5, 'F': 6})
        expected = "{'a': 1, 'B': 2, 'c': '3', 'd': '\"', 'E': 5, 'F': 6}"
        assert_equal(str(nd), expected)

    def test_repr(self):
        assert_equal(repr(NormalizedDict()), 'NormalizedDict()')
        assert_equal(repr(NormalizedDict({'a': None, 'b': '"', 'A': 1})),
                     "NormalizedDict({'a': 1, 'b': '\"'})")
        assert_equal(repr(type('Extend', (NormalizedDict,), {})()), 'Extend()')

    def test_unicode(self):
        nd = NormalizedDict({'a': '\xe4', '\xe4': 'a'})
        assert_equal(str(nd), "{'a': '\xe4', '\xe4': 'a'}")

    def test_update(self):
        nd = NormalizedDict({'a': 1, 'b': 1, 'c': 1})
        nd.update({'b': 2, 'C': 2, 'D': 2})
        for c in 'bcd':
            assert_equal(nd[c], 2)
            assert_equal(nd[c.upper()], 2)
        keys = list(nd)
        assert_true('b' in keys)
        assert_true('c' in keys)
        assert_true('C' not in keys)
        assert_true('d' not in keys)
        assert_true('D' in keys)

    def test_update_using_another_norm_dict(self):
        nd = NormalizedDict({'a': 1, 'b': 1})
        nd.update(NormalizedDict({'B': 2, 'C': 2}))
        for c in 'bc':
            assert_equal(nd[c], 2)
            assert_equal(nd[c.upper()], 2)
        keys = list(nd)
        assert_true('b' in keys)
        assert_true('B' not in keys)
        assert_true('c' not in keys)
        assert_true('C' in keys)

    def test_update_with_kwargs(self):
        nd = NormalizedDict({'a': 0, 'c': 1})
        nd.update({'b': 2, 'c': 3}, b=4, d=5)
        for k, v in [('a', 0), ('b', 4), ('c', 3), ('d', 5)]:
            assert_equal(nd[k], v)
            assert_equal(nd[k.upper()], v)
            assert_true(k in nd)
            assert_true(k.upper() in nd)
            assert_true(k in nd.keys())

    def test_iter(self):
        keys = list('123_aBcDeF')
        nd = NormalizedDict((k, 1) for k in keys)
        assert_equal(list(nd), keys)
        assert_equal([key for key in nd], keys)

    def test_keys_are_sorted(self):
        nd = NormalizedDict((c, None) for c in 'aBcDeFg123XyZ___')
        assert_equal(list(nd.keys()), list('123_aBcDeFgXyZ'))
        assert_equal(list(nd), list('123_aBcDeFgXyZ'))

    def test_keys_values_and_items_are_returned_in_same_order(self):
        nd = NormalizedDict()
        for i, c in enumerate('abcdefghijklmnopqrstuvwxyz0123456789!"#%&/()=?'):
            nd[c.upper()] = i
            nd[c+str(i)] = 1
        assert_equal(list(nd.items()), list(zip(nd.keys(), nd.values())))

    def test_eq(self):
        self._verify_eq(NormalizedDict(), NormalizedDict())

    def test_eq_with_normal_dict(self):
        self._verify_eq(NormalizedDict(), {})

    def test_eq_with_user_dict(self):
        self._verify_eq(NormalizedDict(), UserDict())

    def _verify_eq(self, d1, d2):
        assert_true(d1 == d1 == d2 == d2)
        d1['a'] = 1
        assert_true(d1 == d1 != d2 == d2)
        d2['a'] = 1
        assert_true(d1 == d1 == d2 == d2)
        d1['B'] = 1
        d2['B'] = 1
        assert_true(d1 == d1 == d2 == d2)
        d1['c'] = d2['C'] = 1
        d1['D'] = d2['d'] = 1
        assert_true(d1 == d1 == d2 == d2)

    def test_eq_with_other_objects(self):
        nd = NormalizedDict()
        for other in ['string', 2, None, [], self.test_clear]:
            assert_false(nd == other, other)
            assert_true(nd != other, other)

    def test_ne(self):
        assert_false(NormalizedDict() != NormalizedDict())
        assert_false(NormalizedDict({'a': 1}) != NormalizedDict({'a': 1}))
        assert_false(NormalizedDict({'a': 1}) != NormalizedDict({'A': 1}))

    def test_hash(self):
        assert_raises(TypeError, hash, NormalizedDict())

    def test_clear(self):
        nd = NormalizedDict({'a': 1, 'B': 2})
        nd.clear()
        assert_equal(nd._data, {})
        assert_equal(nd._keys, {})


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.utils.asserts import (assert_almost_equal, assert_equal,
                                 assert_false, assert_none,
                                 assert_not_almost_equal, assert_not_equal,
                                 assert_not_none, assert_raises,
                                 assert_raises_with_msg, assert_true, fail)


AE = AssertionError

class MyExc(Exception):
    pass


class MyEqual:
    def __init__(self, attr=None):
        self.attr = attr
    def __eq__(self, obj):
        try:
            return self.attr == obj.attr
        except AttributeError:
            return False
    def __str__(self):
        return str(self.attr)
    __repr__ = __str__


def func(msg=None):
    if msg is not None:
        raise ValueError(msg)


class TestAsserts(unittest.TestCase):

    def test_assert_raises(self):
        assert_raises(ValueError, int, 'not int')
        self.assertRaises(ValueError, assert_raises, MyExc, int, 'not int')
        self.assertRaises(AssertionError, assert_raises, ValueError, int, '1')

    def test_assert_raises_with_msg(self):
        assert_raises_with_msg(ValueError, 'msg', func, 'msg')
        self.assertRaises(ValueError, assert_raises_with_msg, TypeError, 'msg',
                          func, 'msg')
        try:
            assert_raises_with_msg(ValueError, 'msg', func)
        except AE as err:
            assert_equal(str(err), 'ValueError not raised')
        else:
            raise AssertionError('No AssertionError raised')
        try:
            assert_raises_with_msg(ValueError, 'msg1', func, 'msg2')
        except AE as err:
            expected = "Correct exception but wrong message: msg1 != msg2"
            assert_equal(str(err), expected)
        else:
            raise AssertionError('No AssertionError raised')

    def test_assert_equal(self):
        assert_equal('str', 'str')
        assert_equal(42, 42, 'hello', True)
        assert_equal(MyEqual('hello'), MyEqual('hello'))
        assert_equal(None, None)
        assert_raises(AE, assert_equal, 'str', 'STR')
        assert_raises(AE, assert_equal, 42, 43)
        assert_raises(AE, assert_equal, MyEqual('hello'), MyEqual('world'))
        assert_raises(AE, assert_equal, None, True)

    def test_assert_equal_with_values_having_same_string_repr(self):
        for val, type_ in [(1, 'integer'),
                           (MyEqual(1), 'MyEqual')]:
            assert_raises_with_msg(AE, '1 (string) != 1 (%s)' % type_,
                                   assert_equal, '1', val)
        assert_raises_with_msg(AE, '1.0 (float) != 1.0 (string)',
                               assert_equal, 1.0, '1.0')
        assert_raises_with_msg(AE, 'True (string) != True (boolean)',
                               assert_equal, 'True', True)

    def test_assert_equal_with_custom_formatter(self):
        assert_equal('hyv\xe4', 'hyv\xe4', formatter=repr)
        assert_raises_with_msg(AE, "'hyv\xe4' != 'paha'",
                               assert_equal, 'hyv\xe4', 'paha', formatter=repr)
        assert_raises_with_msg(AE, "'hyv\\xe4' != 'paha'",
                               assert_equal, 'hyv\xe4', 'paha', formatter=ascii)

    def test_assert_not_equal(self):
        assert_not_equal('abc', 'ABC')
        assert_not_equal(42, -42, 'hello', True)
        assert_not_equal(MyEqual('cat'), MyEqual('dog'))
        assert_not_equal(None, True)
        raise_msg = assert_raises_with_msg  # shorter to use here
        raise_msg(AE, "str == str", assert_not_equal, 'str', 'str')
        raise_msg(AE, "hello: 42 == 42", assert_not_equal, 42, 42, 'hello')
        raise_msg(AE, "hello", assert_not_equal, MyEqual('cat'), MyEqual('cat'),
                  'hello', False)

    def test_assert_not_equal_with_custom_formatter(self):
        assert_not_equal('hyv\xe4', 'paha', formatter=repr)
        assert_raises_with_msg(AE, "'\xe4' == '\xe4'",
                               assert_not_equal, '\xe4', '\xe4', formatter=repr)

    def test_fail(self):
        assert_raises(AE, fail)
        assert_raises_with_msg(AE, 'hello', fail, 'hello')

    def test_assert_true(self):
        assert_true(True)
        assert_true('non-empty string is true')
        assert_true(-1 < 0 < 1, 'my message')
        assert_raises(AE, assert_true, False)
        assert_raises(AE, assert_true, '')
        assert_raises_with_msg(AE, 'message', assert_true, 1 < 0, 'message')

    def test_assert_false(self):
        assert_false(False)
        assert_false('')
        assert_false([1,2] == (1,2), 'my message')
        assert_raises(AE, assert_false, True)
        assert_raises(AE, assert_false, 'non-empty')
        assert_raises_with_msg(AE, 'message', assert_false, 0 < 1, 'message')

    def test_assert_none(self):
        assert_none(None)
        assert_raises_with_msg(AE, "message: 'Not None' is not None",
                               assert_none, 'Not None', 'message')
        assert_raises_with_msg(AE, "message",
                               assert_none, 'Not None', 'message', False)

    def test_assert_not_none(self):
        assert_not_none('Not None')
        assert_raises_with_msg(AE, "message: is None",
                               assert_not_none, None, 'message')
        assert_raises_with_msg(AE, "message",
                               assert_not_none, None, 'message', False)

    def test_assert_almost_equal(self):
        assert_almost_equal(1.0, 1.00000001)
        assert_almost_equal(10, 10.01, 1)
        assert_raises_with_msg(AE, 'hello: 1 != 2 within 3 places',
                               assert_almost_equal, 1, 2, 3, 'hello')
        assert_raises_with_msg(AE, 'hello',
                               assert_almost_equal, 1, 2, 3, 'hello', False)

    def test_assert_not_almost_equal(self):
        assert_not_almost_equal(1.0, 1.00000001, 10)
        assert_not_almost_equal(10, 11, 1, 'hello')
        assert_raises_with_msg(AE, 'hello: 1 == 1 within 7 places',
                               assert_not_almost_equal, 1, 1, msg='hello')
        assert_raises_with_msg(AE, 'hi',
                               assert_not_almost_equal, 1, 1.1, 0, 'hi', False)


if __name__ == '__main__':
    unittest.main()



class DupeDynamicKeywords:
    names = ['defined twice', 'DEFINED TWICE',
             'Embedded ${twice}', 'EMBEDDED ${ARG}',
             'Exact dupe is ok', 'Exact dupe is ok']

    def get_keyword_names(self):
        return self.names

    def run_keyword(self, name, args):
        pass



#!/usr/bin/env python3

"""JSON schema for ``robot.running.TestSuite`` model structure.

The schema is modeled using pydantic in this file. After updating the model,
execute this file to regenerate the actual schema file in ``running.json``.

https://pydantic-docs.helpmanual.io/usage/schema/
"""

from collections.abc import Sequence
from pathlib import Path
from typing import Literal

from pydantic import BaseModel, Extra, Field


class BodyItem(BaseModel):
    lineno: int | None
    error: str | None


class Return(BodyItem):
    type = Field('RETURN', const=True)
    values: Sequence[str]


class Continue(BodyItem):
    type = Field('CONTINUE', const=True)


class Break(BodyItem):
    type = Field('BREAK', const=True)


class Error(BodyItem):
    type = Field('ERROR', const=True)
    values: Sequence[str]
    error: str


class Keyword(BodyItem):
    name: str
    args: Sequence[str] | None
    assign: Sequence[str] | None


class For(BodyItem):
    type = Field('FOR', const=True)
    variables: Sequence[str]
    flavor: str
    values: Sequence[str]
    start: str | None
    mode: str | None
    fill: str | None
    body: list['Keyword | For | While | If | Try | Error | Break | Continue | Return']


class While(BodyItem):
    type = Field('WHILE', const=True)
    condition: str | None
    limit: str | None
    on_limit: str | None
    on_limit_message: str | None
    body: list['Keyword | For | While | If | Try | Error | Break | Continue | Return']


class IfBranch(BodyItem):
    type: Literal['IF', 'ELSE IF', 'ELSE']
    condition: str | None
    body: list['Keyword | For | While | If | Try | Error | Break | Continue | Return']


class If(BodyItem):
    type = Field('IF/ELSE ROOT', const=True)
    body: list[IfBranch]


class TryBranch(BodyItem):
    type: Literal['TRY', 'EXCEPT', 'ELSE', 'FINALLY']
    patterns: Sequence[str] | None
    pattern_type: str | None
    variable: str | None
    body: list['Keyword | For | While | If | Try | Error | Break | Continue | Return']


class Try(BodyItem):
    type = Field('TRY/EXCEPT ROOT', const=True)
    body: list[TryBranch]


class TestCase(BaseModel):
    name: str
    doc: str | None
    tags: Sequence[str] | None
    template: str | None
    timeout: str | None
    lineno: int | None
    error: str | None
    setup: Keyword | None
    teardown: Keyword | None
    body: list[Keyword | For | While | If | Try | Error]


class TestSuite(BaseModel):
    """JSON schema for `robot.running.TestSuite`.

    Compatible with JSON Schema Draft 2020-12.
    """
    name: str
    doc: str | None
    metadata: dict[str, str] | None
    source: Path | None
    rpa: bool | None
    setup: Keyword | None
    teardown: Keyword | None
    tests: list[TestCase] | None
    suites: list['TestSuite'] | None
    resource: 'Resource | None'

    class Config:
        # Do not allow extra attributes.
        extra = Extra.forbid
        # pydantic doesn't add schema version automatically.
        # https://github.com/samuelcolvin/pydantic/issues/1478
        schema_extra = {
            '$schema': 'https://json-schema.org/draft/2020-12/schema'
        }


class Import(BaseModel):
    type: Literal['LIBRARY', 'RESOURCE', 'VARIABLES']
    name: str
    args: Sequence[str] | None
    alias: str | None
    lineno: int | None


class Variable(BaseModel):
    name: str
    value: Sequence[str]
    lineno: int | None
    error: str | None


class UserKeyword(BaseModel):
    name: str
    args: Sequence[str] | None
    doc: str | None
    tags: Sequence[str] | None
    return_: Sequence[str] | None
    timeout: str | None
    lineno: int | None
    error: str | None
    body: list[Keyword | For | While | If | Try | Error | Return]


class Resource(BaseModel):
    source: Path | None
    doc: str | None
    imports: list[Import] | None
    variables: list[Variable] | None
    keywords: list[UserKeyword] | None


for cls in [For, While, IfBranch, TryBranch, TestSuite]:
    cls.update_forward_refs()


if __name__ == '__main__':
    path = Path(__file__).parent / 'running.json'
    with open(path, 'w') as f:
        f.write(TestSuite.schema_json(indent=2))
    print(path.absolute())



import unittest

from robot.utils.asserts import assert_equal, assert_raises_with_msg
from robot.model import TestSuite, Keyword
from robot.model.fixture import create_fixture


class TestCreateFixture(unittest.TestCase):

    def test_creates_default_fixture_when_given_none(self):
        suite = TestSuite()
        fixture = create_fixture(suite.fixture_class, None, suite, Keyword.SETUP)
        self._assert_fixture(fixture, suite, Keyword.SETUP)

    def test_sets_parent_and_type_correctly(self):
        suite = TestSuite()
        kw = Keyword('KW Name')
        fixture = create_fixture(suite.fixture_class, kw, suite, Keyword.TEARDOWN)
        self._assert_fixture(fixture, suite, Keyword.TEARDOWN)

    def test_raises_type_error_when_wrong_fixture_type(self):
        suite = TestSuite()
        wrong_kw = object()
        assert_raises_with_msg(
            TypeError, "Invalid fixture type 'object'.",
            create_fixture, suite.fixture_class, wrong_kw, suite, Keyword.TEARDOWN
        )

    def _assert_fixture(self, fixture, exp_parent, exp_type,
                        exp_class=TestSuite.fixture_class):
        assert_equal(fixture.parent, exp_parent)
        assert_equal(fixture.type, exp_type)
        assert_equal(fixture.__class__, exp_class)


if __name__ == '__main__':
    unittest.main()



def get_variables():
    return {
        'scalar_from_cli_varfile'  : ('This variable is not taken into use '
                                      'because it already exists in '
                                      'vars_from_cli.py'),
        'scalar_from_cli_varfile_2': ('Variable from second variable file '
                                      'from cli')
        }






import unittest

from array import array
from collections import UserDict, UserList, UserString
from collections.abc import Mapping
from typing import Any, Dict, List, Optional, Set, Tuple, Union

from robot.utils import (is_bytes, is_falsy, is_dict_like, is_list_like, is_string,
                         is_truthy, is_union, PY_VERSION, type_name, type_repr)
from robot.utils.asserts import assert_equal, assert_true


class MyMapping(Mapping):

    def __getitem__(self, item):
        return 0

    def __len__(self):
        return 0

    def __iter__(self):
        return iter([])


def generator():
    yield 'generated'


class TestIsMisc(unittest.TestCase):

    def test_strings(self):
        for thing in ['string', 'hyvä', '']:
            assert_equal(is_string(thing), True, thing)
            assert_equal(is_bytes(thing), False, thing)

    def test_bytes(self):
        for thing in [b'bytes', bytearray(b'ba'), b'', bytearray()]:
            assert_equal(is_bytes(thing), True, thing)
            assert_equal(is_string(thing), False, thing)

    def test_is_union(self):
        assert is_union(Union[int, str])
        assert is_union(Union[int, str], allow_tuple=True)
        assert not is_union((int, str))
        assert is_union((int, str), allow_tuple=True)
        if PY_VERSION >= (3, 10):
            assert is_union(eval('int | str'))
            assert is_union(eval('int | str'), allow_tuple=True)
        for not_union in 'string', 3, [int, str], list, List[int]:
            assert not is_union(not_union)
            assert not is_union(not_union, allow_tuple=True)


class TestListLike(unittest.TestCase):

    def test_strings_are_not_list_like(self):
        for thing in ['string', UserString('user')]:
            assert_equal(is_list_like(thing), False, thing)

    def test_bytes_are_not_list_like(self):
        for thing in [b'bytes', bytearray(b'bytes')]:
            assert_equal(is_list_like(thing), False, thing)

    def test_dict_likes_are_list_like(self):
        for thing in [dict(), UserDict(), MyMapping()]:
            assert_equal(is_list_like(thing), True, thing)

    def test_files_are_not_list_like(self):
        with open(__file__) as f:
            assert_equal(is_list_like(f), False)
        assert_equal(is_list_like(f), False)

    def test_iter_makes_object_iterable_regardless_implementation(self):
        class Example:
            def __iter__(self):
                1/0
        assert_equal(is_list_like(Example()), True)

    def test_only_getitem_does_not_make_object_iterable(self):
        class Example:
            def __getitem__(self, item):
                return "I'm not iterable!"
        assert_equal(is_list_like(Example()), False)

    def test_iterables_in_general_are_list_like(self):
        for thing in [[], (), set(), range(1), generator(), array('i'), UserList()]:
            assert_equal(is_list_like(thing), True, thing)

    def test_others_are_not_list_like(self):
        for thing in [1, None, True, object()]:
            assert_equal(is_list_like(thing), False, thing)

    def test_generators_are_not_consumed(self):
        g = generator()
        assert_equal(is_list_like(g), True)
        assert_equal(is_list_like(g), True)
        assert_equal(list(g), ['generated'])
        assert_equal(list(g), [])
        assert_equal(is_list_like(g), True)


class TestDictLike(unittest.TestCase):

    def test_dict_likes(self):
        for thing in [dict(), UserDict(), MyMapping()]:
            assert_equal(is_dict_like(thing), True, thing)

    def test_others(self):
        for thing in ['', u'', 1, None, True, object(), [], (), set()]:
            assert_equal(is_dict_like(thing), False, thing)


class TestTypeName(unittest.TestCase):

    def test_base_types(self):
        for item, exp in [('x', 'string'),
                          (u'x', 'string'),
                          (b'x', 'bytes'),
                          (bytearray(), 'bytearray'),
                          (1, 'integer'),
                          (1.0, 'float'),
                          (True, 'boolean'),
                          (None, 'None'),
                          (set(), 'set'),
                          ([], 'list'),
                          ((), 'tuple'),
                          ({}, 'dictionary')]:
            assert_equal(type_name(item), exp)

    def test_file(self):
        with open(__file__) as f:
            assert_equal(type_name(f), 'file')

    def test_custom_objects(self):
        class CamelCase: pass
        class lower: pass
        for item, exp in [(CamelCase(), 'CamelCase'),
                          (lower(), 'lower'),
                          (CamelCase, 'CamelCase')]:
            assert_equal(type_name(item), exp)

    def test_strip_underscores(self):
        class _Foo_: pass
        assert_equal(type_name(_Foo_), 'Foo')

    def test_none_as_underscore_name(self):
        class C:
            _name = None
        assert_equal(type_name(C()), 'C')
        assert_equal(type_name(C(), capitalize=True), 'C')

    def test_typing(self):
        for item, exp in [(List, 'list'),
                          (List[int], 'list'),
                          (Tuple, 'tuple'),
                          (Tuple[int], 'tuple'),
                          (Set, 'set'),
                          (Set[int], 'set'),
                          (Dict, 'dictionary'),
                          (Dict[int, str], 'dictionary'),
                          (Union, 'Union'),
                          (Union[int, str], 'Union'),
                          (Optional, 'Optional'),
                          (Optional[int], 'Union'),
                          (Any, 'Any')]:
            assert_equal(type_name(item), exp)

    if PY_VERSION >= (3, 10):
        def test_union_syntax(self):
            assert_equal(type_name(int | float), 'Union')

    def test_capitalize(self):
        class lowerclass: pass
        class CamelClass: pass
        assert_equal(type_name('string', capitalize=True), 'String')
        assert_equal(type_name(None, capitalize=True), 'None')
        assert_equal(type_name(lowerclass(), capitalize=True), 'Lowerclass')
        assert_equal(type_name(CamelClass(), capitalize=True), 'CamelClass')


class TestTypeRepr(unittest.TestCase):

    def test_class(self):
        class Foo:
            pass
        assert_equal(type_repr(Foo), 'Foo')

    def test_none(self):
        assert_equal(type_repr(None), 'None')

    def test_ellipsis(self):
        assert_equal(type_repr(...), '...')

    def test_string(self):
        assert_equal(type_repr('MyType'), 'MyType')

    def test_no_typing_prefix(self):
        assert_equal(type_repr(List), 'List')

    def test_generics_from_typing(self):
        assert_equal(type_repr(List[Any]), 'List[Any]')
        assert_equal(type_repr(Dict[int, None]), 'Dict[int, None]')
        assert_equal(type_repr(Tuple[int, ...]), 'Tuple[int, ...]')

    if PY_VERSION >= (3, 9):
        def test_generics(self):
            assert_equal(type_repr(list[Any]), 'list[Any]')
            assert_equal(type_repr(dict[int, None]), 'dict[int, None]')

    def test_union(self):
        assert_equal(type_repr(Union[int, float]), 'int | float')
        assert_equal(type_repr(Union[int, None, List[Any]]), 'int | None | List[Any]')
        if PY_VERSION >= (3, 10):
            assert_equal(type_repr(int | None | list[Any]), 'int | None | list[Any]')


class TestIsTruthyFalsy(unittest.TestCase):

    def test_truthy_values(self):
        for item in [True, 1, [False], unittest.TestCase, 'truE', 'whatEver']:
            for item in self._strings_also_in_different_cases(item):
                assert_true(is_truthy(item) is True)
                assert_true(is_falsy(item) is False)

    def test_falsy_values(self):
        class AlwaysFalse:
            __bool__ = __nonzero__ = lambda self: False
        falsy_strings = ['', 'faLse', 'nO', 'nOne', 'oFF', '0']
        for item in falsy_strings + [False, None, 0, [], {}, AlwaysFalse()]:
            for item in self._strings_also_in_different_cases(item):
                assert_true(is_truthy(item) is False)
                assert_true(is_falsy(item) is True)

    def _strings_also_in_different_cases(self, item):
        yield item
        if is_string(item):
            yield item.lower()
            yield item.upper()
            yield item.title()


if __name__ == '__main__':
    unittest.main()
